/*
*	FLYTRI.C	Demonstrate use of SCNEWDEV and GRLINE
*			functions by drawing a moving triangle.
*
*  FLYTRI demonstrates the use of the Turbo C TOOLS line graphics
*  function GRLINE.  It selects a graphics mode (mode 15 or mode
*  4, depending on installed hardware).  Then it draws a colored
*  triangle in many different positions on the screen.
*
*  When it is done, it waits for a key press and then returns to
*  the mode the screen was in when FLYTRI was invoked.
*
*  The command line format is as follows:
*
*	flytri
*
*/


#include <math.h>
#include <stdio.h>

#include <bgraphs.h>
#include <bkeybrd.h>
#include <bscreens.h>


main ()
{
    double rotx, roty, size, x, y;
    int    cols, mode, oldmode, page;
    int    color = 0;
    int    junk;
    int    numcolors;
    PT	   p1, p2, p3;

		/* Get information on which adapter(s) are	    */
		/* installed.					    */
    scequip ();

		/* Find out what adapter we are using now.	    */
    scmode (&oldmode, &cols, &page);

		/* Exit if no graphics adapter exists.		    */
    if ((b_cga == ABSENT && b_ega == ABSENT && b_pgc == ABSENT)  ||
	(((mode = scnewdev (15, 25)) == -1)		       &&
	 ((mode = scnewdev (4, 25)) == -1)))
	{
	    printf ("flytri: can use only modes 4 and 15\n");
	    exit (1);
	}

		/* Set number of available colors dependent on mode */
		/* we are using.				    */
    switch (mode)
    {
	case 15:
	    numcolors = 4;
	    break;

	case 4:
	    numcolors = 4;
    }

		/* Use transcendental functions to move a triangle  */
		/* across the graphics screen while permuting its   */
		/* shape.					    */
    for (x = 0.0;  (int) x < b_maxx - 40;  x += (b_maxx / 80))
    {
	if (++color == numcolors)
	    color = 1;

	y    = utabs (sin ((x / b_maxx) * 3.2)) * (0.70 * b_maxy);
	size = utabs (sin ((x / b_maxx) * 2.4)) * (50.0);

	rotx = sin ((x / b_maxx) * 4.0);
	roty = cos ((y / b_maxy) * 2.9);

	p1.x = x + (rotx * size);
	p1.y = y + (roty * size);
	p2.x = x + (rotx * size);
	p2.y = y - (roty * size);
	p3.x = x - (rotx * size);
	p3.y = y - (roty * size);

		/* Draw lines, corner to corner, to make the sides  */
		/* of the triangle.				    */
	grline (&p1, &p2, color);
	grline (&p2, &p3, color);
	grline (&p3, &p1, color);
    }

		/* Wait for a keystroke.			    */
    kbgetkey (&junk);

		/* Change back to old adapter, mode, and page.	    */
    scnewdev (oldmode, 25);
    scpage (page);
}
