/*
*	MENU.C		Display several types of keyboard-activated
*			menus.
*
*  This program displays a series of menus on the screen, allowing
*  the user to choose which of several types of menus will be
*  displayed.
*
*  Four menus are used.  The main menu allows the user to select
*  one of three other menus or to terminate the program.  The
*  main menu is an example of a vertical menu style; the other
*  three exhibit horizontal, grid, and Lotus-style menus,
*  respectively.
*
*  The command line format is as follows:
*
*	menu
*
*  The main thrusts of MENU are to show off the new menu-ing
*  capabilities of Turbo C TOOLS, and to provide a working example
*  of the proper method of construction and use of the menu
*  functions.
*
*/

#include <ctype.h>
#include <stdio.h>

#include <bkeybrd.h>
#include <bkeys.h>
#include <bmenu.h>
#include <butil.h>

		/* Text color is intense white on blue. 	    */
#define MYTEXTATR (utnybbyt (BLUE,  WHITE   | INTENSITY))

		/* "Lotus" description color is intense magenta on  */
		/* blue.					    */
#define MYLNGATTR (utnybbyt (BLUE,  MAGENTA | INTENSITY))

		/* Color of highlight bar is black text on a white  */
		/* background.					    */
#define MYHILATR  (utnybbyt (WHITE, BLACK))

		/* Color of "protected" items is green on blue.     */
#define MYPROATR  (utnybbyt (BLUE,  GREEN))

		/* Color of menu borders is cyan on black.	    */
#define MYBORDATR (utnybbyt (BLACK, CYAN))

		/* Color of menu titles is the same as menu borders.*/
#define MYTITATR  MYBORDATR

		/* A few important constants.			    */
#define NUL    '\0'
#define TRUE   1
#define FALSE  0

		/* Terminate-on-error macro.			    */
#define exitBad()						     \
    {								     \
	fprintf (stderr,					     \
		 "menu: died with b_wnerr = %d in line %d",          \
		 b_wnerr, __LINE__);				     \
	exit (b_wnerr); 					     \
    }


		/* Declare internal subroutines.		    */
void horizontal (int, int, int, int *, int *);
void lotus	(int, int, int, int *, int *);
void grid	(int, int, int, int *, int *);


void main ()
{
    BMENU   *pmenu;
    BORDER   border;
    WHERE    where;
    int      ch, scan;
    int      row, col;
    int      rrow, rcol;
    int      showrow, showcol;
    int      adapter, mode, cols, apage;
    int      done = FALSE;
    int      coff, crow, ccol, chigh, clow;

		/* First we make certain that we are in 80 column   */
		/* text video mode.				    */
    adapter = scmode (&mode, &cols, &apage);
    switch (mode)
    {
	case 2:
	case 3:
	case 7:
	    break;

	default:
	    fprintf (stderr,
		     "menu: This demonstration works only in 80 column\n");
	    fprintf (stderr,
		     "      text modes (modes 2, 3, and 7)\n");
	    exit (1);
    }

		/* Save cursor position and style to restore later. */
    coff = sccurst (&crow, &ccol, &chigh, &clow);

		/* Create the menu data structure.		    */
    if ((pmenu = mncreate (7, 14,
			   MYTEXTATR, MYHILATR,
			   MYPROATR, MYLNGATTR)) == NIL)
	exitBad ()

		/* Set up items on the menu, and define 	    */
		/* corresponding keys (upper and lower case of the  */
		/* first letters of the items). 		    */
    if (mnitmkey (pmenu, 0, 1, 0, "Horizontal", "Hh", MN_NOMOVE) == NIL)
	exitBad ()
    if (mnitmkey (pmenu, 1, 1, 0, "Lotus-Style", "Ll", MN_NOMOVE) == NIL)
	exitBad ()
    if (mnitmkey (pmenu, 2, 1, 0, "Grid", "Gg", MN_NOMOVE) == NIL)
	exitBad ()
    if (mnitmkey (pmenu, 4, 1, 0, "Quit", "QqXx", MN_NOMOVE | MN_BEEP) == NIL)
	exitBad ()

		/* Define the following keys as selection &	    */
		/* transmission keys: ALT-H, ALT-L, ALT-G, and	    */
		/* ALT-Q and X. 				    */
    if (mnkey (pmenu,  0,  1, KB_C_A_H, KB_S_A_H,
	       MN_SELECT | MN_TRANSMIT, MN_ADD) == NIL)
	exitBad ()
    if (mnkey (pmenu,  1,  1, KB_C_A_L, KB_S_A_L,
	       MN_SELECT | MN_TRANSMIT, MN_ADD) == NIL)
	exitBad ()
    if (mnkey (pmenu,  2,  1, KB_C_A_G, KB_S_A_G,
	       MN_SELECT | MN_TRANSMIT, MN_ADD) == NIL)
	exitBad ()
    if (mnkey (pmenu,  4,  1, KB_C_A_Q, KB_S_A_Q,
	       MN_SELECT | MN_TRANSMIT | MN_BEEP, MN_ADD) == NIL)
	exitBad ()
    if (mnkey (pmenu,  4,  1, KB_C_A_X, KB_S_A_X,
	       MN_SELECT | MN_TRANSMIT | MN_BEEP, MN_ADD) == NIL)
	exitBad ()

		/* Disable ESC key.				    */
    if (mnkey (pmenu, 0, 0, KB_C_N_ESC, KB_S_N_ESC,
	       MN_ABORT, MN_DELETE) == NIL)
	exitBad ()

		/* Now that we have put all of the menu items on    */
		/* the menu in item color, set the native window    */
		/* color so that additional text will appear in the */
		/* menu's window in "long-item (description)" color.*/
    wnsetopt (pmenu->pwin, WN_ATTR, MYLNGATTR);

		/* Figure out where to display the menu.	    */
    where.dev	     = (adapter == 0) ? MONO : COLOR;
    where.page	     = 0;
    where.corner.row = 1;
    where.corner.col = 1;

		/* Make a border with a top centered title.	    */
    border.type     = BBRD_SSSS | BBRD_TCT;
    border.attr     = MYBORDATR;
    border.ch	    = NUL;
    border.pttitle  = "Menu Styles";
    border.ttattr   = MYTITATR;

		/* Display the menu on the screen.		    */
    if (mndsplay (pmenu, &where, &border) == NIL)
	exitBad ()

		/* Set up starting row and column for highlight bar.*/
    row = 0;
    col = 1;

		/* Leave this menu on the screen until they select  */
		/* the "Quit" entry.                                */
    do
    {		/* Read a response from the menu.		    */
	if (mnread (pmenu, row, col, &row, &col, &ch, &scan,
		    MN_KEEP_HIGHLIGHT))
	    exitBad ()

		/* Set up location to show sub-menu.		    */
	showrow = where.corner.row + row + 2;
	showcol = where.corner.col + col + 0;

		/* Go do what was requested.			    */
	switch (row)
	{
	    case 0:
		horizontal (where.dev, showrow, showcol, &rrow, &rcol);
		break;
	    case 1:
		lotus (where.dev, showrow, showcol, &rrow, &rcol);
		break;
	    case 2:
		grid (where.dev, showrow, showcol, &rrow, &rcol);
		break;
	    case 4:
		done = TRUE;
	}

		/* At this point, rrow and rcol contain the row and */
		/* column of the item selected from the submenu.    */
	if (!done)
	{	/* Tell the user what was selected from the submenu.*/
	    if (b_pcurwin != pmenu->pwin)
		wnselect (pmenu->pwin);
	    wnscrblk (b_pcurwin, 6, 0, 6, 13, -1, -1, 0, 0, 0);
	    wncurmov (6, 0);
	    wnprintf ("xmit (%d %d)", rrow, rcol);
	    wnupblk (b_pcurwin, 6, 0, 6, 13);
	}
    } while (!done);

    mndstroy (pmenu);

		/* Restore cursor position and style.		    */
    sccurset (crow, ccol);
    scpgcur (coff, chigh, clow, CUR_NO_ADJUST);
}



/**
*
* Name		HORIZONTAL -- Display and allow user selection from
*			      a simple horizontal menu.
*
* Synopsis	horizontal (dev, row, col, prrow, prcol);
*
*		int dev 	  Device on which to display the
*				  menu.  Either COLOR or MONO.
*		int row, col	  Row and column where the upper-
*				  left corner of the menu's data
*				  area should appear.
*		int *prrow,	  Pointers to variables in which to
*		    *prcol	  return the row and column selected
*				  from the menu.
*
* Description	This function constructs a simple horizontal menu,
*		and waits for user input.  It then returns the row
*		and column of the selection to its caller.
*
* Returns	*prrow, *prcol	  Row and column (relative to menu) of
*				  user selection.
*
**/


void horizontal (dev, row, col, prrow, prcol)
int  dev;
int  row,    col;
int *prrow, *prcol;
{
    BMENU  *pmenu;
    BORDER  border;
    WHERE   where;
    int     ch, scan;

		/* Figure out where to display the menu.	    */
    where.dev	     = dev;
    where.page	     = 0;
    where.corner.row = row;
    where.corner.col = col;

		/* Create the menu data structure.		    */
    if ((pmenu = mncreate (1, 28,
			   MYTEXTATR, MYHILATR,
			   MYPROATR, MYLNGATTR)) == NIL)
	exitBad ()

		/* Set up items on the menu, and add keys to the key*/
		/* binding list (upper and lower case of the first  */
		/* letters of the items.			    */
    if (mnitmkey (pmenu, 0, 1,	0, "My dog", "MmDd",   MN_NOMOVE) == NIL)
	exitBad ()
    if (mnitmkey (pmenu, 0, 9,	0, "has", "Hh",   MN_NOMOVE) == NIL)
	exitBad ()
    if (mnitmkey (pmenu, 0, 14, 0, "itchy", "Ii",   MN_NOMOVE) == NIL)
	exitBad ()
    if (mnitmkey (pmenu, 0, 21, 0, "fleas.", "Ff",   MN_NOMOVE) == NIL)
	exitBad ()

		/* Disable ESC key.				    */
    if (mnkey (pmenu, 0, 0, KB_C_N_ESC, KB_S_N_ESC,
	       MN_ABORT, MN_DELETE) == NIL)
	exitBad ()

		/* Make a border with a bottom centered title.	    */
    border.type     = BBRD_SSSS | BBRD_BCT;
    border.attr     = MYBORDATR;
    border.ch	    = NUL;
    border.pbtitle  = "Horizontal Menu";
    border.btattr   = MYTITATR;

		/* Display the menu on the screen.		    */
    if (mndsplay (pmenu, &where, &border) == NIL)
	exitBad ()

		/* Read a response from the menu.		    */
    if (mnread (pmenu, 0, 1, prrow, prcol, &ch, &scan, MN_REMOVE))
	exitBad ()
}




/**
*
* Name		LOTUS -- Display and allow user selection from a
*			 "Lotus"-style menu.
*
* Synopsis	lotus (dev, row, col, prrow, prcol);
*
*		int dev 	  Device on which to display the
*				  menu.  Either COLOR or MONO.
*		int row, col	  Row and column where the upper-
*				  left corner of the menu's data
*				  area should appear.
*		int *prrow,	  Pointers to variables in which to
*		    *prcol	  return the row and column selected
*				  from the menu.
*
* Description	This function constructs a "Lotus" menu, and
*		waits for user input.  It then returns the row
*		and column of the selection to its caller.
*
* Returns	*prrow, *prcol	  Row and column (relative to menu) of
*				  user selection.
*
**/


void lotus (dev, row, col, prrow, prcol)
int  dev;
int  row,    col;
int *prrow, *prcol;
{
    BMENU  *pmenu;
    BORDER  border;
    WHERE   where;
    int     ch, scan;

		/* Figure out where to display the menu.	    */
    where.dev	     = dev;
    where.page	     = 0;
    where.corner.row = row;
    where.corner.col = col;

		/* Create the menu data structure.		    */
    if ((pmenu = mncreate (2, 70,
			   MYTEXTATR, MYHILATR,
			   MYPROATR, MYLNGATTR)) == NIL)
	exitBad ()

		/* Set up items on the menu, and add keys to the key*/
		/* binding list (upper and lower case of the first  */
		/* letters of the items.			    */
    if (mnlitkey (pmenu, 0, 0, 0, "Worksheet", 1, 0,
	"Global, Insert, Delete, Column-Width, Erase, Titles, Window, Status",
	"Ww",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

    if (mnlitkey (pmenu, 0, 11, 0, "Range", 1, 0,
	"Format, Label-Prefix, Erase, Name, Justify, Protect, Unprotect, Input",
	"Rr",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

    if (mnlitkey (pmenu, 0, 18, 0, "Copy", 1, 0,
	"Copy a cell or range of cells",
	"Cc",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

    if (mnlitkey (pmenu, 0, 24, 0, "Move", 1, 0,
	"Move a cell or range of cells",
	"Mm",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

    if (mnlitkey (pmenu, 0, 30, 0, "File", 1, 0,
	"Retrieve, Save, Combine, Xtract, Erase, List, Import, Directory",
	"Ff",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

    if (mnlitkey (pmenu, 0, 36, 0, "Print", 1, 0,
	"Output a range to the printer or a print file",
	"Pp",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

    if (mnlitkey (pmenu, 0, 43, 0, "Graph", 1, 0,
	"Create a graph",
	"Gg",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

    if (mnlitkey (pmenu, 0, 50, 0, "Data", 1, 0,
	"Fill, Table, Sort, Query, Distribution",
	"Dd",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

    if (mnlitkey (pmenu, 0, 56, 0, "Quit", 1, 0,
	"End 3-2-1 Session (Have you saved your play ?)",
	"Qq",   MN_NOMOVE | MN_TRANSMIT) == NIL)
	exitBad ()

		/* Disable ESC key.				    */
    if (mnkey (pmenu, 0, 0, KB_C_N_ESC, KB_S_N_ESC,
	       MN_ABORT, MN_DELETE) == NIL)
	exitBad ()

		/* Make a border with a top centered title.	    */
    border.type     = BBRD_SDDD | BBRD_TCT;
    border.attr     = MYBORDATR;
    border.ch	    = NUL;
    border.pttitle  = "Lotus Menu  (with top-centered title)";
    border.ttattr   = MYTITATR;

		/* Display the menu on the screen.		    */
    if (mndsplay (pmenu, &where, &border) == NIL)
	exitBad ()

		/* Read a response from the menu.		    */
    if (mnlread (pmenu, 0, 0, prrow, prcol, &ch, &scan, MN_DESTROY))
	exitBad ()
}




/**
*
* Name		GRID -- Display and allow user selection from a
*			grid menu.
*
* Synopsis	grid (dev, row, col, prrow, prcol);
*
*		int dev 	  Device on which to display the
*				  menu.  Either COLOR or MONO.
*		int row, col	  Row and column where the upper-
*				  left corner of the menu's data
*				  area should appear.
*		int *prrow,	  Pointers to variables in which to
*		    *prcol	  return the row and column selected
*				  from the menu.
*
* Description	This function constructs a grid menu, and
*		waits for user input.  It then returns the
*		row and column of the selection to its caller.
*
* Returns	*prrow, *prcol	  Row and column (relative to menu) of
*				  user selection.
*
**/


void grid (dev, row, col, prrow, prcol)
int  dev;
int  row,    col;
int *prrow, *prcol;
{
    BMENU  *pmenu;
    BORDER  border;
    WHERE   where;
    int     ch, scan, x, y;
    char    s[3];

		/* Figure out where to display the menu.	    */
    where.dev	     = dev;
    where.page	     = 0;
    where.corner.row = row;
    where.corner.col = col;

		/* Create the menu data structure.		    */
    if ((pmenu = mncreate (6, 40,
			   MYTEXTATR, MYHILATR,
			   MYPROATR, MYLNGATTR)) == NIL)
	exitBad ()

		/* Set up items on the menu.  We will make a grid   */
		/* of numbers from 0 to 47, in rows across the	    */
		/* menu.					    */
    for (y = 0;  y < 6;  y++)
	for (x = 0;  x < 8;  x++)
		/* Put a the ASCII characters for a number (y*8+x)  */
		/* at location (y, x*5) in the menu.		    */
	    if (mnitem (pmenu, y, (x * 5), 0,
			itoa (((y * 8) + x), s, 10)) == NIL)
		 exitBad ()

		/* Disable ESC key.				    */
    if (mnkey (pmenu, 0, 0, KB_C_N_ESC, KB_S_N_ESC,
	       MN_ABORT, MN_DELETE) == NIL)
	exitBad ()

		/* Make a border with a bottom left title.	    */
    border.type     = BBRD_SSSS | BBRD_BLT;
    border.attr     = MYBORDATR;
    border.ch	    = NUL;
    border.pbtitle  = "Grid Menu  (bottom left title)";
    border.btattr   = MYTITATR;

		/* Display the menu on the screen.		    */
    if (mndsplay (pmenu, &where, &border) == NIL)
	exitBad ()

		/* Read a response from the menu.		    */
    if (mnread (pmenu, 0, 0, prrow, prcol, &ch, &scan, MN_DESTROY))
	exitBad ()
}
