/**
*
*  BINTERV.H   Header file for TURBO C TOOLS Intervention Functions
*
*  Version 5.00  (C)Copyright Blaise Computing Inc.  1987
*
**/

#ifndef DEF_BINTERV		      /* Prevent redefinition.	      */

typedef struct		      /* IV_KEY:  Specifies one hot key.      */
{			      /*				      */
    char ch;		      /* Character value.		      */
    char keycode;	      /* Key code (pseudo-scan code).	      */
    int  action;	      /* Action to be taken when this key is  */
			      /*  detected (see possibilities below). */
} IV_KEY;

			      /* Key actions			      */
			      /*				      */
#define IV_KY_NONE	  0   /* No hot key pressed.		      */
#define IV_KY_SERVICE	  1   /* Invoke intervention function.	      */
#define IV_KY_SLEEP	  2   /* Put intervention function to sleep.  */
#define IV_KY_WAKE	  3   /* Awaken intervention function.	      */

typedef struct		      /* IV_TIME:  Specifies time of day or   */
{			      /*  time interval for intervention code.*/
			      /*				      */
    long ticks; 	      /* Clock ticks since midnight or	      */
			      /*  between invocations.		      */
    int  action;	      /* Type of time event (moment or	      */
			      /*  interval).			      */
} IV_TIME;

			      /* Time actions			      */
			      /*				      */
#define IV_TM_NONE	  0   /* No time event occurred.	      */
#define IV_TM_INTERVAL	  1   /* Regular interval between invocations.*/
#define IV_TM_MOMENT	  2   /* Time of day.			      */

typedef struct		      /* IV_EVENT:  Event that triggered the  */
{			      /*  intervention function.	      */
			      /*				      */
    IV_KEY key; 	      /* Hot key found, if any. 	      */
			      /*				      */
    int  time_action;	      /* Type of timer event (moment or       */
			      /*  interval), if any.		      */
    int  time_index;	      /* Index of timer event in table.       */
    long time;		      /* Current time of day, measured in     */
			      /*  clock ticks since midnight.	      */
} IV_EVENT;

typedef struct		      /* IV_VECTORS:  set of interrupt	      */
{			      /*  vectors used by intervention code.  */
			      /*				      */
    void far *ptimer;	      /* INT 08h:  Timer tick.		      */
    void far *pkeybd;	      /* INT 09h:  Keystroke.		      */
    void far *pdisk;	      /* INT 13h:  BIOS disk services.	      */
    void far *pdos;	      /* INT 21h:  DOS functions.	      */
    void far *pidle;	      /* INT 28h:  DOS idle.		      */
} IV_VECTORS;

typedef struct		      /* IV_CTRL:  intervention control block */
{			      /*				      */
    unsigned int signature;   /* Signature identifying this structure.*/
    unsigned int sign2;       /* One's complement of "signature".     */
    char	 ident[16];   /* Identifying name.		      */
    unsigned int psp;	      /* PSP of this program.		      */
    int 	 enabled;     /* Nonzero if enabled (this prevents the*/
			      /*  filters from being installed twice).*/
    void   far	*pgate;       /* Gateway for invoking this program    */
			      /*  from outside. 		      */
    IV_VECTORS	prev_vec;     /* Previous values of interrupt vectors.*/
			      /*				      */
    char   dos_need;	      /* Nonzero if intervention function     */
			      /*  uses DOS services.		      */
    char   dkey_need;	      /* Nonzero if intervention function     */
			      /*  uses DOS functions 1-12.	      */
			      /*				      */
    IV_KEY far *pkeytab;      /* Table of hot keys.		      */
    int    ktsize;	      /* Size of hot key table. 	      */
			      /*				      */
    IV_KEY key_event;	      /* Found key, if any.		      */
			      /*				      */
			      /* Internal state of filters:	      */
			      /*				      */
    char   wait;	      /* Nonzero if INT 21h or INT 28h filter */
			      /*  is cycling, waiting for the	      */
			      /*  intervention function to be invoked.*/
    char   req; 	      /* Nonzero if intervention function     */
			      /*  needs invocation.		      */
    char   awake;	      /* Zero if intervention function is     */
			      /*  asleep.			      */

			      /* Busy flags:			      */
    char   timer_busy;	      /*  INT 08h filter.		      */
    char   kb_busy;	      /*  INT 09h filter.		      */
    char   disk_busy;	      /*  INT 13h filter.		      */
    char   idle_busy;	      /*  INT 28h filter.		      */
			      /*				      */
    char   idle_safe;	      /* Nonzero if INT 0x28 is in progress   */
			      /*  and DOS functions are available     */
			      /*  (but not functions 1-12).	      */
			      /*				      */
    void   far *pschedblk;    /* Entry point into scheduler's ISR     */
			      /*  control block.		      */
			      /*				      */
    unsigned char	      /* Address of DOS critical section flag */
	      far *pdos_crit; /*  (busy flag).			      */
			      /*				      */
    char   _reserved[8];      /* Reserved for future use.	      */
} IV_CTRL;

#define IV_SIGNATURE   (('0' << 8) | 'B')   /* "B0"                   */

    /* Option values for IVINSTAL.				      */

#define IV_DOS_NEED	 1    /* Intervention function uses DOS.      */
#define IV_NO_DOS_NEED	 0    /* Intervention function doesn't use    */
			      /*  DOS.				      */

#define IV_DKEY_NEED	 2    /* Intervention function uses DOS       */
			      /*  functions 1-12.		      */
#define IV_NO_DKEY_NEED  0    /* Intervention function doesn't use    */
			      /*  DOS functions 1-12.		      */

#define IV_FLOAT_NEED	 4    /* Intervention point uses floating     */
			      /*  point interrupt vectors while other */
			      /*  programs do so also.		      */
#define IV_NO_FLOAT_NEED 0    /*				      */

    /* Option values for IVVECS.				      */

#define  IV_RETVEC  1	      /* Return current values of interrupt   */
			      /*  vectors.			      */
#define  IV_SETVEC  0	      /* Set interrupt vectors. 	      */

    /* Symbols for error codes					      */

#define  IV_NO_ERROR	   0
#define  IV_NOT_ENABLED    1
#define  IV_NOT_INSTALLED  2
#define  IV_NULL_POINTER   3
#define  IV_BAD_OPT	   4
#define  IV_INSTALLED	   5

    /* Function declarations					      */

				      /* Install intervention function*/
int cdecl ivinstal(void (*)(IV_EVENT *),
		   char *,	      /*			      */
		   char *,int,	      /*			      */
		   IV_KEY *,int,      /*			      */
		   IV_TIME *,int,     /*			      */
		   int);	      /*			      */
				      /*			      */
IV_CTRL far * cdecl ivsense(	      /* Detect presence of	      */
	IV_VECTORS far *,char far *); /* intervention function.       */
				      /*			      */
int cdecl ivdisabl(IV_CTRL far *);    /* Disable intervention function*/
				      /*			      */
int cdecl ivvecs(int,		      /* Set or return interrupt      */
		 IV_VECTORS far *);   /* vectors used by the	      */
				      /* intervention filters.	      */
				      /*			      */
IV_CTRL far * cdecl ivctrl(void);     /* Return address of this       */
				      /* program's intervention       */
				      /* control block. 	      */

#define DEF_BINTERV	 1	      /* Prevent second reading of    */
				      /* these definitions.	      */

#endif				      /* Ends "#ifndef DEF_BINTERV"   */
