/**
*
*  BKEYBRD.H	Header file for BIOS keyboard functions for TURBO C TOOLS
*
*  Version	5.00 (C)Copyright Blaise Computing Inc.  1986, 1987
*
**/

#ifndef DEF_BKEYBRD		/* Prevent redefinition.	    */

#include <ctype.h>
#include <butil.h>

#define KB_BIOS_INT 0x16	/* General BIOS keyboard gate.	    */


#define KB_HEAD     0		/* KBSTUFF, KBPLACE: Whether to     */
				/* place char(s) at HEAD or at	    */
#define KB_TAIL     1		/* TAIL of queue.		    */

				/* KBPLACE error returns:	    */
#define KB_OK	    0		/*   no error,			    */
#define KB_FULL     1		/*   buffer full,		    */
#define KB_PLACE    2		/*   bad "where" specification.     */

#define KB_DATASEG  0x40	/* Segment value of BIOS keyboard   */
				/* data.			    */
#define KB_HEADLOC  0x1a	/* Location of queue head pointer.  */
#define KB_TAILOC   0x1c	/* Location of queue tail pointer.  */
				/*				    */
#define KB_SHIFTLOC 0x17	/* Location of shift status register*/
				/*				    */
#define KB_BUFEND   0x3e	/* Address of keyboard buffer end.  */
#define KB_BUFSTART 0x1e	/* Address of keyboard buffer start.*/

				/* Size of buffer in bytes.	    */
#define KB_BUFSIZE    (KB_BUFEND - KB_BUFSTART)

				/* Size of buffer in keystrokes.    */
#define KB_BUFACSIZE  (((KB_BUFEND - KB_BUFSTART) >> 1) - 1)

				/* Far pointer to buffer head word. */
#define KB_BUFHEADADDR (uttofaru (KB_DATASEG, KB_HEADLOC))

				/* Far pointer to buffer tail word. */
#define KB_BUFTAILADDR (uttofaru (KB_DATASEG, KB_TAILOC))

				/* Far pointer to shift status byte.*/
#define KB_SHIFTADDR (uttofaru (KB_DATASEG, KB_SHIFTLOC))


/* Symbol for KBSET and KBSTATUS -- Maps keyboard status.	    */
/* register onto unsigned integer.				    */
typedef struct kstatus		/* Keyboard status structure.	    */
{				/*				    */
   unsigned right_shift  : 1;	/* Right Shift key depressed.	    */
   unsigned left_shift	 : 1;	/* Left  Shift key depressed.	    */
   unsigned ctrl_shift	 : 1;	/* Ctrl        key depressed.	    */
   unsigned alt_shift	 : 1;	/* Alt	       key depressed.	    */
   unsigned scroll_state : 1;	/* Scroll Lock has been toggled.    */
   unsigned num_state	 : 1;	/* Num Lock has been toggled.	    */
   unsigned caps_state	 : 1;	/* Caps Lock has been toggled.	    */
   unsigned ins_state	 : 1;	/* Insert state is active.	    */
				/*				    */
   unsigned filler	 : 3;	/* Filler for word alignment.	    */
   unsigned hold_state	 : 1;	/* Suspend key has been toggled.    */
   unsigned scroll_shift : 1;	/* Scroll Lock key depressed.	    */
   unsigned num_shift	 : 1;	/* Num Lock    key depressed.	    */
   unsigned caps_shift	 : 1;	/* Caps Lock   key depressed.	    */
   unsigned ins_shift	 : 1;	/* Insert      key depressed.	    */
} KEYSTATUS;



/* Function declarations.					    */

int   cdecl kbready (char *, int *);	/* Check for a keystroke.   */
					/* Get a character from CON */
char  cdecl kbquery (char *,int,int *,int *);
int   cdecl kbgetkey (int *);		/* Await & return char and  */
					/*   scan code. 	    */
int   cdecl kbflush (void);		/* Flush keyboard buffer.   */
int   cdecl kbqueue (int *);		/* Total and remaining in   */
					/*   keyboard queue.	    */
void  cdecl kbset (KEYSTATUS *);	/* Set shift status.	    */
char *cdecl kbstuff (int, char *);	/* Stuff string into queue. */
int   cdecl kbplace (int, char, char);	/* Place one key into queue.*/


#define kbstatus(pkeybd) ((*((int *) (pkeybd))) = utpeekw (KB_SHIFTADDR))

#define kbscanof(c) (isascii ((int) c)				     \
		    ? ((int) b_keycod [((int) (c))])		     \
		    : (-1))

#define kbcharof(c) (isascii ((int) c)				     \
		    ? ((int) (c))				     \
		    : (-1))


extern unsigned char b_keycod [];


#define DEF_BKEYBRD 1		/* Prevent second reading of these  */
				/* definitions. 		    */

#endif				/* Ends "#ifndef DEF_BKEYBRD"       */
