/**
*
*  BMENU.H	Header file for Turbo C TOOLS Menu Functions
*
*  Version	5.00 (C)Copyright Blaise Computing Inc.  1987
*
**/


#ifndef DEF_BMENU		/* Prevent redefinition.	    */

#include <bwindow.h>


/* Error constants specific to MENU operations. 		    */
#define MN_BAD_MENU	100	/* Menu failed test bu MNVALMNU.    */
#define MN_NONE_SELECT	101	/* No selectable item on menu.	    */
#define MN_KEY_CONFLICT 102	/* A key not bound to a location    */
				/* already exists with the given    */
				/* character and key codes.	    */
#define MN_NO_ITEM	103	/* No item exists at the given	    */
				/* coordinates. 		    */
#define MN_BAD_KEY	104	/* Invalid keymap entry encountered.*/
#define MN_BAD_ITEM	105	/* Invalid item encountered.	    */
#define MN_NO_EXIT	106	/* No enabled key has either the    */
				/* transmit or abort bit set.	    */

#define MN_READ_AB	110	/* User pressed a key with MN_ABORT */
				/* action associated with it.	    */
#define MN_UNKNOWN_AB	111	/* User pressed a key unknown to    */
				/* the menu, and MN_UNKNOWN_TRANSMIT*/
				/* was an option passed to a menu   */
				/* reader function.		    */


#define MN_KEY_SIGN  0x0123	/* Arbitrary signature words for    */
#define MN_ITEM_SIGN 0xfedc	/* keymap and item list entries.    */

#define MN_DEAD_KEY  0xffef	/* Arbitrary "deleted" signature    */
#define MN_DEAD_ITEM 0xeefe	/* words for same.		    */


/* Constants for use with MNKEY (the "howchange" argument).         */
#define MN_ADD	   0
#define MN_CHANGE  1
#define MN_DELETE  2

/* Constants for use with MNKEY (the "action" argument).            */
#define MN_NOMOVE	 0x000
#define MN_UP		 0x001
#define MN_DOWN 	 0x002
#define MN_RIGHT	 0x003
#define MN_LEFT 	 0x004
#define MN_NEXT 	 0x005
#define MN_PREVIOUS	 0x006
#define MN_FIRST	 0x007
#define MN_LAST 	 0x008
#define MN_SELECT	 0x009

#define MN_TRANSMIT	 0x010
#define MN_BEEP 	 0x020
#define MN_TEMP_DISABLE  0x040
#define MN_DISABLE	 0x080
#define MN_ABORT	 0x100

#define MN_MOVE_MASK 0x00f
#define MNMOVE(action) (action & MN_MOVE_MASK)

#define MN_INVALID_ACTION 0x4f3


/* Constants for use with adding/changing of items.		    */
#define MN_PROTECT   0x001
#define MN_NOPROTECT 0x000


/* Constants for use with MNREAD (the "option" value).              */
#define MN_REMOVE	     0x01
#define MN_DESTROY	     0x03
#define MN_KEEP_HIGHLIGHT    0x04
#define MN_ALL_TRANSMIT      0x08
#define MN_UNKNOWN_BEEP      0x10
#define MN_UNKNOWN_TRANSMIT  0x20
#define MN_KEEP_DESCRIPTION  0x40
#define MN_HOLD_BEEP	     0x80


/* Definitive signature for signature field in BMENU structure.     */
#define BMENU_SIGN "BCI TCTmnu 5.00"


/* Internal Macros.						    */
#define mnvalidm(pmenu) 					     \
    {								     \
	if (mnvalmnu (pmenu) == NIL)				     \
	    wnreterr (MN_BAD_MENU);				     \
    }

#define mntrunc(pmenu, pstring, col)				     \
		((strlen (pstring) + col > pmenu->pwin->img.dim.w)   \
		? (pmenu->pwin->img.dim.w - col)		     \
		: (strlen (pstring)))


/* Definitions of data types.					    */

		/* BITEM -- data structure for an item in a menu.   */
typedef struct bitem
{
    unsigned signature; 	/* Characteristic signature.	    */
    int      row, col;		/* Coordinates (relative to window) */
				/* of menu item text.		    */
    int      lrow, lcol;	/* Row and column at which to put   */
				/* long description.		    */
    int      attr;		/* Attribute of this item.	    */
    int      len;		/* Length of ASCII text of menu     */
				/* item.			    */
    int      llen;		/* Length of ASCII text of menu     */
				/* description. 		    */
    int      option;		/* Bitwise OR-ing of:		    */
				/*   MN_PROTECT (protected item)    */
				/*   MN_NOPROTECT (unprotected item)*/
    char    *plstring;		/* Long description of item.  Used  */
				/* by "Lotus" menu functions.       */
    struct   bitem *next;	/* Pointer to next item 	    */
} BITEM;			/* specification in linked list.    */


		/* BKEYMAP -- data structure for a key associated   */
		/*	      with a menu.			    */
typedef struct bkeymap
{
    unsigned signature; 	/* Characteristic signature.	    */
    int      ch;		/* Character and scan code with     */
    int      scan;		/* which the following action is to */
				/* be associated with.		    */
				/*				    */
    int      row, col;		/* The row and column this keycode  */
				/* is associated with.		    */
				/*				    */
    int      action;		/* The action which should be taken */
				/* when (ch,scan) is received by    */
				/* MNREAD:			    */
				/*				    */
				/* The bitwise OR-ing of the	    */
				/* following:			    */
				/*     MN_TRANSMIT, MN_BEEP,	    */
				/*     MN_SELECT, MN_DISABLE,	    */
				/*     and MN_ABORT,		    */
				/*				    */
				/*   OR-ed with only one of:	    */
				/*				    */
				/*     MN_UP, MN_DOWN, MN_RIGHT,    */
				/*     MN_LEFT, MN_NEXT,	    */
				/*     MN_PREVIOUS, MN_FIRST,	    */
				/*     MN_LAST, or MN_NOMOVE.	    */
				/*				    */
    struct   bkeymap *next;	/* Pointer to next key in keymap.   */
} BKEYMAP;


		/* BKEYTAB -- data structure for a default key table*/
		/*	      entry.				    */
typedef struct bkeytab
{
    int      ch;		/* Character and scan code with     */
    int      scan;		/* which the following action is to */
				/* be associated with.		    */
				/*				    */
    int      action;		/* The action which should be taken */
				/* when (ch,scan) is received by    */
				/* MNREAD: (see above). 	    */
} BKEYTAB;


		/* BMENU -- Data structure which governs behavior   */
		/*	    and appearance of a menu.		    */
typedef struct
{
    char       signature[16];	/* Characteristic signature.	    */
				/*				    */
    BWINDOW   *pwin;		/* The window we use.		    */
				/*				    */
    BITEM     *pitems;		/* The items to display in the	    */
				/* window.			    */
				/*				    */
    BKEYMAP   *pkeys;		/* The key definitions used.	    */
				/*				    */
    int        hilattr; 	/* Highlight bar attributes.	    */
    int        proattr; 	/* Protected attributes.	    */
    int        longattr;	/* Description attributes.	    */
} BMENU;


		/* Pointer to current default key map.		    */
extern BKEYTAB *b_mnkeytab;

		/* Original default key map.			    */
extern BKEYTAB b_mndefkey [];


/* Function declarations -- User functions.			    */
BMENU *cdecl mncreate (int,	/* Create menu data structures,     */
		 int, int, int, /* set up default key bindings and  */
		 int, int);	/* attributes.			    */
				/*				    */
BMENU *cdecl mndsplay (BMENU *, /* Display a menu.  Necessary	    */
		 WHERE *,	/* before using MNREAD. 	    */
		 BORDER *);	/*				    */
				/*				    */
int cdecl mndstroy (BMENU *);	/* Release menu data structures,    */
				/* undisplay if necessary.	    */
				/*				    */
BMENU *cdecl mnitem (BMENU *,	/* Add/Change/Delete item in menu.  */
		 int, int, int, /*				    */
		 char *);	/*				    */
				/*				    */
BMENU *cdecl mnitmkey (BMENU *, /* Add item and key bindings for    */
		 int, int, int, /* that item to menu.		    */
		 char *,	/*				    */
		 char *, int);	/*				    */
				/*				    */
BMENU *cdecl mnkey (BMENU *,	/* Add a key binding to a menu.     */
		 int, int, int, /*				    */
		 int, int, int);/*				    */
				/*				    */
BMENU *cdecl mnlitem  (BMENU *, /* Add/Change/Delete item in	    */
		 int, int, int, /* "Lotus"-style menu.              */
		 char *,	/*				    */
		 int, int,	/*				    */
		 char *);	/*				    */
				/*				    */
BMENU *cdecl mnlitkey (BMENU *, /* Add "Lotus" item and key         */
		 int, int, int, /* bindings for that item to menu.  */
		 char *,	/*				    */
		 int, int,	/*				    */
		 char *,	/*				    */
		 char *, int);	/*				    */
				/*				    */
int cdecl mnlread (BMENU *, int,/* Request user input from a	    */
		  int, int *,	/* "Lotus"-style menu.              */
		  int *, int *, /*				    */
		  int *, int);	/*				    */
				/*				    */
int cdecl mnread (BMENU *, int, /* Request user input from a menu.  */
		 int, int *,	/*				    */
		 int *, int *,	/*				    */
		 int *, int);	/*				    */
				/*				    */


/* Function declarations -- Internal functions. 		    */
BMENU *cdecl mnatr (BMENU *,	/* Change attributes on an item in  */
	      BITEM *, int);	/* a menu.			    */
				/*				    */
BMENU *cdecl mndefkey (BMENU *);/* Set up default keys for a menu.  */
				/* Internal function -- redefine if */
				/* you want different default key   */
				/* bindings.			    */
				/*				    */
BMENU *cdecl mndisabl (BMENU *);/* Disable all keys which are bound */
				/* to locations for which no items  */
				/* exist -- disable unbound keys.   */
				/*				    */
int cdecl mndlitms (BMENU *);	/* Delete all items from a menu     */
				/* item list data structure.	    */
				/*				    */
int cdecl mndlkeys (BMENU *);	/* Delete all keys from a menu key  */
				/* list data structure. 	    */
				/*				    */
BITEM *cdecl mnfindsl (BMENU *, /* Find the first selectable item   */
		 BITEM *,	/* in a menu, given starting	    */
		 int, int,	/* coordinates and menu item.	    */
		 int *);	/*				    */
				/*				    */
BITEM *cdecl mnmchitm (BMENU *, /* Match a menu item to a (row,col) */
		 BITEM *,	/* specification.		    */
		 int, int,	/*				    */
		 int, int *);	/*				    */
				/*				    */
BKEYMAP *cdecl mnmchkey (	/* Match a menu keymap entry to a   */
		   BMENU *,	/* (ch,scan) specification.	    */
		   BKEYMAP *,	/*				    */
		   int, int,	/*				    */
		   int *);	/*				    */
				/*				    */
int cdecl mnreadx (BMENU *,	/* Request user input from a menu.  */
		 BITEM **,	/*				    */
		 BKEYMAP **,	/*				    */
		 int *, int *,	/*				    */
		 int);		/*				    */
				/*				    */
BMENU *cdecl mnvalmnu (BMENU *);/* Validate a menu data structure   */
				/* (really checks for wild pointers)*/


#define DEF_BMENU  1		/* Prevent second reading of	    */
				/* these definitions.		    */

#endif				/* Ends "#ifndef DEF_BMENU"         */
