/**
*
*  BSCREENS.H	Header file for TURBO C TOOLS Screen Functions
*
*  Version 5.00 (C) Copyright Blaise Computing Inc. 1983, 1987
*
**/

#ifndef DEF_BSCREENS		      /* Prevent redefinition.	      */

#include <butil.h>

#define  SC_BIOS_INT  16	      /* BIOS interrupt for screen    */
				      /* functions.		      */

#define  NORMAL     7		      /* Common attributes for the    */
#define  REVERSE    112 	      /* monochrome adapter	      */
#define  UNDERLINE  1
#define  INTENSITY  8
#define  MONOBLINK  128

#define  BLACK	    0		      /* (Byte) values representing   */
#define  BLUE	    1		      /* color attributes.	      */
#define  GREEN	    2
#define  CYAN	    3
#define  RED	    4
#define  MAGENTA    5
#define  BROWN	    6
#define  WHITE	    7

    /* Maximum dimensions of usual PC screens			      */

#define  PC_ROWS      25
#define  PC_COLS      80
#define  LAST_ROW     (PC_ROWS - 1)
#define  LAST_COL     (PC_COLS - 1)
#define  PC_BIG_ROWS  43
#define  MAX_DEVICES  2 	      /* Color and/or monochrome      */
#define  MAX_PAGES    8 	      /* Pages per video adapter      */
				      /* (8 maximum on EGA)	      */

    /* Scrolling directions					      */

#define  SCR_UP      0
#define  SCR_DOWN    1
#define  SCR_RIGHT   0
#define  SCR_LEFT    1

    /* Options for SCPGCUR					      */

#define  CUR_ADJUST	1     /* Adjust cursor size into normal       */
			      /* limits if appropriate		      */
#define  CUR_NO_ADJUST	0     /* Use requested cursor scan lines      */
			      /* without any adjustment 	      */

    /* Options for writing strings to screen			      */

#define  CUR_BEG     0	      /* Leave cursor at beginning of string. */
#define  CUR_AFTER   1	      /* Leave cursor after end of string.    */
			      /*				      */
#define  CHARS_ONLY  0	      /* Buffer contains characters only.     */
#define  CHAR_ATTR   2	      /* Buffer contains (char,attr) pairs.   */
			      /*				      */
#define  MOVE_CUR    0	      /* Leave cursor according to	      */
			      /*   CUR_BEG/CUR_AFTER bit.	      */
#define  NO_MOVE_CUR 4	      /* Preserve cursor location.	      */

    /* Options for SCBLINK					      */

#define  SC_BLINK    1	      /* Attribute bit 7 == foreground blink. */
#define  SC_INTENSE  0	      /* Attribute bit 7 == background	      */
			      /*   intensity.			      */

    /* Symbols used by SCEQUIP to indicate the state of the various   */
    /* video adapters						      */

#define  DONT_KNOW  (-1)      /* We haven't checked yet               */
#define  ABSENT     (-2)
#define  MONO	    0
#define  COLOR	    1

#define  HIFUNC     2	      /* PGC in high-function graphics mode   */
			      /*   (not CGA emulation mode)	      */

    /* Global variables set by SCEQUIP indicating installed video     */
    /* hardware and switch settings				      */

extern int b_mdpa;	      /* Monochrome Display & Printer Adapter */
			      /*   DONT_KNOW, ABSENT, or MONO	      */
			      /*				      */
extern int b_cga;	      /* Color/Graphics Monitor Adapter       */
			      /*   DONT_KNOW, ABSENT, or COLOR	      */
			      /*				      */
extern int b_ega;	      /* Enhanced Graphics Adapter	      */
			      /*   DONT_KNOW, ABSENT, MONO or COLOR   */
			      /*				      */
extern int b_pgc;	      /* Professional Graphics Controller     */
			      /*   DONT_KNOW, ABSENT, COLOR or HIFUNC */
			      /*				      */
extern int b_mem_ega;	      /* Amount of memory installed on EGA in */
			      /* 1024-byte units:  64, 128, 256       */
			      /*				      */
extern unsigned b_sw_ega;     /* Switch settings on EGA 	      */
			      /* Low-order bit set means SW1 off, etc.*/
			      /* (Bit value 1 implies that	      */
			      /* corresponding switch is off.)	      */

    /* Global variables for managing multiple devices & display      **/
    /* pages							      */

extern int b_curpage;		      /* Current display page	      */
extern int b_device;		      /* Current adapter:  DONT_KNOW, */
				      /*   MONO, or COLOR	      */

extern int b_know_hw;		      /* Flag stating whether we have */
				      /* yet run SCEQUIP:  0 if no, 1 */
				      /* if yes.		      */

typedef struct			      /* CUR_TYPE structure:	      */
{				      /* high and low scan lines      */
    int high,low;		      /* in a cursor.		      */
} CUR_TYPE;

    /* Internal structures for preserving BIOS video variables	      */

typedef struct
{
    int  known; 		      /* 1 if this structure has      */
				      /*  valid data, 0 if not.       */
    int  curpage;		      /* Copy of b_curpage.	      */
    int  home_x,home_y; 	      /* Graphics home position.      */
    int  maxx,maxy;		      /* Graphics screen dimensions.  */
				      /*			      */
    char pc_area[30];		      /* Standard PC/XT area.	      */
    int  reset_flag;		      /* EGA reset flag.	      */
    char ega_area[5];		      /* Some EGA data. 	      */
    char far *save_ptr; 	      /* Pointer to EGA variables.    */
    char ega_status;		      /* EGA status byte.	      */
} VIDEO_STATE;
extern VIDEO_STATE b_adap_state[2];   /* One entry for MONO, one for  */
				      /* COLOR. 		      */

    /* TURBO C TOOLS functions implemented as macros		      */

#define  scpage(page)  (b_curpage = (page))

    /* Function declarations					      */

char cdecl scequip(void);	      /* Sense hardware environment   */
				      /*			      */
int cdecl  scnewdev(int,int);	      /* Reset and select 25 or 43    */
				      /* line font		      */
				      /*			      */
int cdecl  scchgdev(int);	      /* Select adapter 	      */
				      /*			      */
int cdecl  scmode(int *,int *,int*);  /* Return the video mode info   */
				      /*			      */
int cdecl  scrows(void);	      /* Number of rows on screen     */
				      /*			      */
int cdecl  scpages(void);	      /* Number of pages in this mode */
				      /*			      */
int cdecl  scapage(int);	      /* Display (activate) a page    */
				      /*			      */
void cdecl scpclr(void);	      /* Clear the current page       */
				      /*			      */
int cdecl scclrmsg(int,int,int);      /* Clear a message	      */
				      /*			      */
int cdecl  sccurset(int,int);	      /* Set the cursor position      */
				      /*			      */
int cdecl  scpgcur(int,int,int,int);  /* Alter the cursor size on     */
				      /* current page.		      */
				      /*			      */
int cdecl  sccurst(int *,int *,int *, /* Return position & size of    */
		   int *);	      /* cursor on current page.      */
				      /*			      */
char cdecl scread(int *,int *);       /* Read character and attribute */
				      /*			      */
int cdecl  scattrib(int,int,char,     /* Set the display attributes   */
		    unsigned);	      /*			      */
				      /*			      */
int cdecl  scwrite(char,unsigned);    /* Write copies of a character  */
				      /*			      */
int cdecl  scttywrt(char,int);	      /* Write character TTY-style    */
				      /*			      */
int cdecl  scbox(int,int,int,int,     /* Draw a box		      */
		 int,char,int);       /*			      */
				      /*			      */
void cdecl scttywin(int,int,int,int,  /* Write character TTY-style to */
	      char,int,int,int,int);  /* rectangular region.	      */
				      /*			      */
void cdecl scwrap(int,int,int,int,    /* Write to rectangular region  */
	    int,char *,int,int,int);  /* with word wrap 	      */
				      /*			      */
int cdecl  scblink(int);	      /* Choose role of attribute bit */
				      /* 7:  foreground blink or      */
				      /* background intensity.	      */
				      /*			      */
int cdecl  scpalett(char *);	      /* Load EGA palette registers.  */
				      /*			      */
int cdecl  scpal1(unsigned,unsigned); /* Load individual EGA palette  */
				      /* register.		      */
				      /*			      */
int cdecl  scborder(unsigned);	      /* Set screen border color.     */
				      /*			      */
int cdecl  scmode4(int,int);	      /* Set mode 4 palette and       */
				      /* background color.	      */

#define DEF_BSCREENS 1		      /* Prevent second reading of    */
				      /* these definitions.	      */

#endif				      /* Ends "#ifndef DEF_BSCREENS"  */
