/**
*
*  BWINDOW.H	Header file for Turbo C TOOLS Window Functions
*
*  Version	5.00 (C)Copyright Blaise Computing Inc.  1986, 1987
*
**/


#ifndef DEF_BWINDOW		/* Prevent redefinition.	    */

#include <bscreens.h>		/* Turbo C TOOLS Screen functions.  */

#define WN_EXT_ERROR 1		/* 1 if wnerror() is external, 0 if */
				/* it's a macro.                    */



/********************************************************************/
/* Definitions of data types.					    */
/********************************************************************/


typedef struct			/* LOC structure:		    */
{				/* row and column relative to a     */
    int row,col;		/* rectangular region.		    */
} LOC;


typedef struct			/* REGION structure:		    */
{				/*   rectangular region on a	    */
				/*   screen.			    */
    LOC ul;			/* Upper left  corner.		    */
    LOC lr;			/* Lower right corner.		    */
} REGION;


typedef struct			/* CELL structure:		    */
{				/* character and attrib. byte	    */
    char ch,attr;		/* on a PC screen.		    */
} CELL;


typedef struct			/* DIM structure:		    */
{				/*   height and width of a	    */
    int h,w;			/*   rectangular region.	    */
} DIM;


typedef struct			/* IMAGE structure:		    */
{				/*   a copy of a rectangular	    */
				/*   portion of a screen's data.    */
    DIM  dim;			/* Dimensions of the rectangle.     */
    CELL *pdata;		/* Pointer to array of cells	    */
				/*   containing the data, row	    */
				/*   by row.			    */
} IMAGE;


typedef struct			/* WHERE structure:		    */
{				/* device, page, and location	    */
				/* where a window may be shown.     */
				/*				    */
    int  dev;			/* Device where shown:	COLOR,	    */
				/* MONO, or ABSENT if not shown.    */
				/*				    */
    int  page;			/* Video page number:  0 to 7	    */
				/*				    */
    LOC  corner;		/* Upper left corner of region	    */
				/* where window data is shown	    */
				/* (does not include border).	    */
} WHERE;


typedef struct bjoint {
    unsigned char  row; 	/* Row and column of joint relative */
    unsigned char  col; 	/* to upper-left of border.  (0,0). */
    char	   jtype;	/* IBM graphic char for joint.	    */
    char	   _dummy;	/* Structure alignment. 	    */
    struct bjoint *next;	/* Pointer to next joint in linked  */
} BJOINT;			/* list of joints.		    */


typedef struct {
    int     type;		/* Type of border:		    */
				/*    0 = no border.		    */
				/*   31 = Character "ch" border.    */
				/*				    */
				/*	  T R B L		    */
				/*	  o t o f		    */
				/*	  p   t t		    */
				/*				    */
				/*    1 = S S S S no title.	    */
				/*    2 = S S S D no title.	    */
				/*    3 = D S S S no title.	    */
				/*    4 = D S S D no title.	    */
				/*    5 = S D S S no title.	    */
				/*    6 = S D S D no title.	    */
				/*    7 = D D S S no title.	    */
				/*    8 = D D S D no title.	    */
				/*    9 = S S D S no title.	    */
				/*   10 = S S D D no title.	    */
				/*   11 = D S D S no title.	    */
				/*   12 = D S D D no title.	    */
				/*   13 = S D D S no title.	    */
				/*   14 = S D D D no title.	    */
				/*   15 = D D D S no title.	    */
				/*   16 = D D D D no title.	    */
				/*				    */
				/* OR the above with:		    */
				/*   0x0020- top-left-title	    */
				/*   0x0040- top-center-title	    */
				/*   0x0080- top-right-title	    */
				/*   0x0100- bottom-left-title	    */
				/*   0x0200- bottom-center-title    */
				/*   0x0400- bottom-right-title     */
				/*				    */
				/* OR the above with:		    */
				/*   0x0800- border has joints.     */
				/*				    */
    int     attr;		/* Attribute of border. 	    */
    char    ch; 		/* Border character (if type==0).   */
    char    _dummy;		/* Structure alignment. 	    */
    char   *pttitle;		/* Title on top border. 	    */
    char   *pbtitle;		/* Title on bottom border.	    */
    int     ttattr;		/* Attributes of top title.	    */
    int     btattr;		/* Attributes of bottom title.	    */
    BJOINT *pjoints;		/* Pointer to linked list of joints.*/
} BORDER;


typedef struct			/* BWINDOW structure:		    */
{				/* everything needed to control     */
				/* an individual window.	    */
				/*				    */
    char   signature[16];	/* Identifying signature.	    */
				/*				    */
    LOC    cur_loc;		/* Location of window's own         */
				/* cursor relative to window's      */
				/* data area.			    */
				/*				    */
    CUR_TYPE cur_type;		/* Cursor type. 		    */
				/*				    */
    IMAGE  img; 		/* Contents of data area.	    */
				/*				    */
    WHERE  where_shown; 	/* Where window is currently	    */
				/* shown.			    */
				/*				    */
    IMAGE  prev;		/* Previous contents of screen	    */
				/* (before window was shown),	    */
				/* including data under border.     */
				/*				    */
    WHERE  where_prev;		/* Region occupied by window	    */
				/* (including border).		    */
				/*				    */
    struct bwin_node *pnode;	/* Pointer to window node	    */
				/* among all windows on a video     */
				/* display page.		    */
				/*				    */
    BORDER bord;		/* Border description.		    */
				/*				    */
    int    attr;		/* Default attributes for data	    */
				/* area.			    */

    struct			/* "options" substructure:          */
    {				/* items set by user request.	    */
				/*				    */
	unsigned delayed   : 1; /* Output postponed until next	    */
				/* update.			    */
				/*				    */
	unsigned cur_off   : 1; /* Cursor invisible.		    */
				/*				    */
	unsigned removable : 1; /* Removable (hence previous	    */
				/* contents of screen must be	    */
				/* preserved).			    */
				/*				    */
	unsigned hidden    : 1; /* Invisible, yet attached to a     */
				/* location on a display page.	    */
				/*				    */
	unsigned _dummy    :12; /* Pad to word boundary 	    */
    } options;

    struct			/* "internals" substructure:        */
    {				/* not directly set by user.	    */
				/*				    */
	unsigned frozen     :1; /* Updates postponed		    */
				/* involuntarily.		    */
				/*				    */
	unsigned dirty	    :1; /* Output request(s) are	    */
				/* awaiting update.		    */
				/*				    */
	unsigned		/* Some portion of data area is     */
	    any_data_covered:1; /* covered by another window.	    */
				/*				    */
	unsigned cur_frozen :1; /* Not selected for visible	    */
				/* cursor.			    */
				/*				    */
	unsigned temp_hid   :1; /* Temporarily hidden by	    */
				/* internal operations. 	    */
				/*				    */
	unsigned _dummy    :11; /* Pad to word boundary 	    */
    } internals;
} BWINDOW;


typedef struct bwin_node	/* WIN_NODE structure:		    */
{				/* node in the linked lists which   */
				/* govern the hierarchy of windows  */
				/* on each display page.	    */
				/*				    */
    char      signature[16];	/* Identifying signature.	    */
				/*				    */
    struct bwin_node *above;	/* Window above this one (or	    */
				/* NIL if this is top window).	    */
				/*				    */
    struct bwin_node *below;	/* Window below this one (or	    */
				/* NIL if this is bottom window).   */
				/*				    */
    BWINDOW	     *pwin;	/* BWINDOW structure belonging	    */
				/* to this node.		    */
} WIN_NODE;



/********************************************************************/
/* Internal Windows Macros.					    */
/********************************************************************/

		/* Signature to identify genuine BWINDOW structure. */
#define BWIN_SIGN "BCI TCTwin 5.00"

		/* Signature to identify genuine WIN_NODE structure.*/
#define BNODE_SIGN "BCI TCTnod 5.00"

				/* Height of a REGION		    */
#define REGION_H(region) ((region).lr.row - (region).ul.row + 1)

				/* Width of a REGION	      */
#define REGION_W(region) ((region).lr.col - (region).ul.col + 1)

#define BBRD_HASJOINTS 0x800
#define BBRD_TYPE      0x01f
#define BBRD_TITLE     0x7e0

#define B_WNPRBF_SIZE  1024

#define wnreterr(errnum) return ((wnerror (errnum)), NIL)
#define wnreterz(errnum) return ((wnerror (errnum)), 0)
#define wnretern(errnum) return (wnerror (errnum))

#define wnreterv(errnum)					     \
    {								     \
	wnerror (errnum);					     \
	return; 						     \
    }

#define wnvalidn(pnode) 					     \
    {								     \
	if (wnvalnod (pnode) == NIL)				     \
	    wnreterr (WN_BAD_NODE);				     \
    }

#define wnvalidw(pwin)						     \
    {								     \
	if (wnvalwin (pwin) == NIL)				     \
	    wnreterr (WN_BAD_WIN);				     \
    }

		/* Construct a new attribute from a given attribute */
		/* and fore and back, either of which may be -1.    */
#define wndoattr(attr, fore, back)				     \
	utnybbyt ((back) == -1 ? uthinyb (attr) : (back),	     \
		  (fore) == -1 ? utlonyb (attr) : (fore))



/********************************************************************/
/* Error codes. 						    */
/********************************************************************/

#define WN_NO_ERROR	    0
#define WN_NO_MEMORY	    1
#define WN_ILL_DIM	    2
#define WN_NULL_PTR	    3
#define WN_BAD_WIN	    4
#define WN_BAD_DEV	    5
#define WN_BAD_PAGE	    6
#define WN_BAD_NODE	    7
#define WN_ALREADY_SHOWN    8
#define WN_NOT_SHOWN	    9
#define WN_CANT_HIDE	   10
#define WN_COVERED	   11
#define WN_BAD_OPT	   12
#define WN_HARD 	   13
#define WN_ILL_VALUE	   14
#define WN_PR_OVERFLOW	   15



/********************************************************************/
/* Constants (type of borders and title(s)) for WNPUTBOR/WNDSPLAY.  */
/********************************************************************/

#define BBRD_NO_BORDER 0x000
#define BBRD_CHAR      0x01f

		/* The letters in the part after the underscore in  */
		/* the following definitions are symbolic names for */
		/* line styles.  A border style with double lines   */
		/* on the top and left, and single lines on the     */
		/* right and bottom would be BBRD_DSDS.  The order  */
		/* of sides in the definition is clockwise from the */
		/* top. 					    */
#define BBRD_SSSS      0x001
#define BBRD_SSSD      0x002
#define BBRD_DSSS      0x003
#define BBRD_DSSD      0x004
#define BBRD_SDSS      0x005
#define BBRD_SDSD      0x006
#define BBRD_DDSS      0x007
#define BBRD_DDSD      0x008
#define BBRD_SSDS      0x009
#define BBRD_SSDD      0x00a
#define BBRD_DSDS      0x00b
#define BBRD_DSDD      0x00c
#define BBRD_SDDS      0x00d
#define BBRD_SDDD      0x00e
#define BBRD_DDDS      0x00f
#define BBRD_DDDD      0x010

		/* The letters in the part after the underscore in  */
		/* the following definitions are symbolic names for */
		/* title styles.  A title style with a top centered */
		/* title would be BBRD_TCT; one with a bottom right */
		/* title would be BBRD_BRT, and so on.		    */
#define BBRD_TLT       0x020
#define BBRD_TCT       0x040
#define BBRD_TRT       0x080
#define BBRD_BLT       0x100
#define BBRD_BCT       0x200
#define BBRD_BRT       0x400



/********************************************************************/
/* Constants (direction of scroll) for WNSCRBLK.		    */
/********************************************************************/

#define WNSCR_UP    0
#define WNSCR_DOWN  1
#define WNSCR_RIGHT 2
#define WNSCR_LEFT  3



/********************************************************************/
/* Constants (option OR values) for WNWRSTRN.			    */
/********************************************************************/

#define WN_NO_UPDATE 0x04
#define WN_UPDATE    0x00
#define WN_LF_LF     0x08
#define WN_LF_CRLF   0x00
#define WN_NO_SCROLL 0x10
#define WN_SCROLL    0x00



/********************************************************************/
/* Window control items & status items for WNGETOPT & WNSETOPT.     */
/* Negative item codes require the window to be currently	    */
/* displayed (although it may be hidden).			    */
/********************************************************************/

#define WN_DEVICE	     1	/* Where the window is		    */
#define WN_PAGE 	   (-2) /* displayed			    */
#define WN_ROW_CORNER	   (-3) /*				    */
#define WN_COL_CORNER	   (-4) /*				    */
				/*				    */
#define WN_HIDDEN	   (-5) /* Whether it's hidden.             */
				/*				    */
#define WN_CUR_OFF	     6	/* Cursor on/off state		    */
#define WN_CUR_HIGH	     7	/* Cursor scan lines		    */
#define WN_CUR_LOW	     8	/*				    */

				/* User-controllable options	    */
#define WN_DELAYED	     9	/*				    */
#define WN_REMOVABLE	    10	/*				    */

#define WN_FROZEN	    11	/* Internal effects of		    */
#define WN_DIRTY	    12	/*   displaying windows 	    */
#define WN_ANY_DATA_COVERED 13	/*				    */

#define WN_ROWS 	    14	/* Dimensions of data area	    */
#define WN_COLS 	    15	/*				    */
				/*				    */
#define WN_ROW_REL	    16	/* Cursor location relative to	    */
#define WN_COL_REL	    17	/* data area			    */
				/*				    */
#define WN_ROW_ABS	  (-18) /* Absolute cursor location on	    */
#define WN_COL_ABS	  (-19) /* screen			    */

#define WN_ROW_OVERALL	  (-27) /* Location of window with	    */
#define WN_COL_OVERALL	  (-28) /* border.			    */
#define WN_HT_OVERALL	  (-29) /* Dimensions of window with	    */
#define WN_WID_OVERALL	  (-30) /* existing border.		    */
				/*				    */
#define WN_ATTR 	    20	/* Default attributes for data	    */
				/* area 			    */
				/*				    */
#define WN_BOR_TYPE	    22	/* Border type			    */
#define WN_BOR_CHAR	    23	/* Border character		    */
#define WN_BOR_ATTR	    24	/* Border attribute		    */
#define WN_BOR_TTATTR	    32	/* Border top title attribute	    */
#define WN_BOR_BTATTR	    33	/* Border bottom title attribute    */
				/*				    */
#define WN_IS_CURRENT	    25	/* Whether this is the current	    */
				/* window			    */
				/*				    */
#define WN_ACTIVE_CUR	  (-26) /* Whether this window (among	    */
				/* all the windows displayed on     */
				/* its page) is the one whose	    */
				/* cursor is active.		    */
				/*				    */
#define WN_PREV_ALLOC	    31	/* Whether space has been allocated */
				/* for the text underneath this     */
				/* window (when displayed and not   */
				/* hidden).			    */



/********************************************************************/
/* User Macros. 						    */
/********************************************************************/

		/* Write a character to the current window	    */
		/* TTY-style.					    */
#define wnwrtty(ch, fore, back) 				     \
	wnwrttyx (b_pcurwin, ch, fore, back,			     \
		  WN_UPDATE | WN_LF_LF)

		/* Write a string to the current window TTY-style   */
		/* with scrolling and end-of-line character wrap.   */
#define wnwrstr(pbuffer, fore, back)				     \
	wnwrstrn (b_pcurwin, pbuffer, 0, fore, back,		     \
		  WN_UPDATE | CHARS_ONLY);



/********************************************************************/
/* Global variables						    */
/********************************************************************/

extern int	b_wnerr;	/* Most recent error		    */
				/* (WN_NO_ERROR if none).	    */

extern BWINDOW *b_pcurwin;	/* Window selected for I/O	    */
				/* (NIL if none).		    */



/********************************************************************/
/* Global variables for internal use only			    */
/********************************************************************/

extern char    *b_wnprbf;	/* Buffer allocated for WNPRINTF.   */

extern unsigned int b_wnbfsz;	/* Size of buffer allocated for     */
				/* WNPRINTF.			    */

				/* List of windows displayed on     */
				/* each page (NIL if none).	    */
extern WIN_NODE *(b_wnlist[MAX_DEVICES][MAX_PAGES]);

				/* The window (on each page)	    */
				/* selected to have a visible	    */
				/* cursor (NIL if none).	    */
extern WIN_NODE *(b_pactnode[MAX_DEVICES][MAX_PAGES]);



/********************************************************************/
/* User functions declarations by category.			    */
/********************************************************************/
		/* Window creation and destruction.		    */

BWINDOW *cdecl wncreate (int,	/* Allocate & create BWINDOW	    */
			 int,	/* structure.			    */
			 int);	/*				    */
				/*				    */
int cdecl wndstroy (BWINDOW *); /* Detach from screen & discard     */
				/* memory structures (does not	    */
				/* alter screen).		    */


		/* Controlling/retrieving window options & status.  */

BWINDOW *cdecl wnsetopt (	/* Set option.			    */
		 BWINDOW *,	/*				    */
		 int, int);	/*				    */
				/*				    */
BWINDOW *cdecl wngetopt (	/* Retrieve option or status.	    */
		 BWINDOW *,	/*				    */
		 int,int *);	/*				    */


		/* Error reporting.				    */

#if WN_EXT_ERROR		/*				    */
				/* External version:		    */
int cdecl wnerror(int); 	/* Set or clear error code and	    */
				/* return it.			    */
#else				/* Macro version:		    */
#define wnerror(ercode) \	/*				    */
	  (b_wnerr = (ercode))	/*				    */
#endif				/*				    */


		 /* Displaying/removing windows.		    */

BWINDOW *cdecl wndsplay (	/* Display window with border.	    */
		 BWINDOW *,	/*				    */
		 WHERE *,	/*				    */
		 BORDER *);	/*				    */
				/*				    */
BWINDOW *cdecl wnremove (	/* Remove window from display page. */
		 BWINDOW *);	/*				    */
				/*				    */
BWINDOW *cdecl wnupdate (	/* Satisfy pending output requests. */
		 BWINDOW *);	/*				    */
				/*				    */
int cdecl wnredraw (int, int);	/* Redisplay all windows on a	    */
				/* display page and restore cursor. */


		/* Cursor action.				    */

BWINDOW *cdecl wncursor (	/* Select for visible cursor	    */
		 BWINDOW *);	/* (deselect other windows shown    */
				/* on this display page.)	    */
				/*				    */
BWINDOW *cdecl wncurmov (int,	/* Move current window's cursor     */
			 int);	/* relative to the window's data    */
				/* area.			    */
				/*				    */
BWINDOW *cdecl wncurpos (int *, /* Return current window's cursor   */
			 int *);/* position relative to the	    */
				/* window's data area.              */


		/* Miscellaneous.				    */

BWINDOW *cdecl wnselect (	/* Select window for I/O (deselect  */
		 BWINDOW *);	/* all other windows) making this   */
				/* the "current" window.            */
				/*				    */
unsigned wnsetbuf(unsigned);	/* Make scratch buffer for WNPRINF. */


		/* Window output.				    */

void cdecl wnwrap (int, char *, /* Write a string TTY-style with    */
		   int, int,	/* word wrap.			    */
		   int);	/*				    */
				/*				    */
void cdecl wnwrstrn (BWINDOW *, /* Write a string of a given length */
		     char *,	/* TTY-style with end-of-line	    */
		     int, int,	/* character wrap.		    */
		     int, int); /*				    */
				/*				    */
int cdecl wnwrbuf (int, int,	/* Write a buffer to a given	    */
		  int,char *,	/* location in the current window's */
		  int, int,	/* data area.			    */
		  int); 	/*				    */
				/*				    */
int wnprintf (char *, ...);	/* Do a PRINTF to the current	    */
				/* window.			    */


		/* Window input.				    */

int cdecl wnrdbuf (int, int,	/* Read part of the current	    */
		   int, char *, /* window's data area into a        */
		   int);	/* buffer.			    */
				/*				    */
char cdecl wnquery (char *,	/* Return input from operator.	    */
		    int,int *); /*				    */


		/* Window scrolling.				    */

BWINDOW *cdecl wnscrblk (	/* Scroll a block-section of a	    */
		 BWINDOW *,	/* window a specified number of     */
		 int, int, int, /* rows/columns.		    */
		 int, int, int, /*				    */
		 int, int, int);/*				    */
				/*				    */
BWINDOW *cdecl wnscroll (	/* Vertically scroll current window.*/
		 int, int,	/*				    */
		 int, int);	/*				    */
				/*				    */
BWINDOW *cdecl wnhoriz (	/* Horizontally scroll current	    */
		 int, int,	/* window.			    */
		 int, int);	/*				    */
				/*				    */


		/* Attribute manipulation.			    */

BWINDOW *cdecl wnatrblk (	/* Change attributes on a block-    */
		 BWINDOW *,	/* section of a window. 	    */
		 int, int, int, /*				    */
		 int, int, int, /*				    */
		 int);		/*				    */
				/*				    */
BWINDOW *cdecl wnatrstr (	/* Change attributes on a stream-   */
		 BWINDOW *,	/* section of a window. 	    */
		 int, int, int, /*				    */
		 int, int,	/*				    */
		 int);		/*				    */
				/*				    */
BWINDOW *cdecl wnattr ( 	/* Change attributes on current     */
		 int,int);	/* window (but don't change         */
				/* default).			    */



/********************************************************************/
/* Internal functions by category				    */
/********************************************************************/

		/* Device & page switching.			    */

int cdecl wnseldev(WHERE *,DIM *,int *);


		/* Error detection.				    */

BWINDOW *cdecl wnvalwin(BWINDOW *);
WIN_NODE *cdecl wnvalnod(WIN_NODE *);


		/* Handling IMAGE structures.			    */

IMAGE *cdecl wnputimg (IMAGE *,WHERE *);
IMAGE *cdecl wnpimblk (IMAGE *,WHERE *,int,int,int,int);
IMAGE *cdecl wngetimg (IMAGE *,WHERE *);


		/* Manipulating chains of WIN_NODEs.		    */

WIN_NODE *cdecl wnpgadd(BWINDOW *,int,int);
BWINDOW  *cdecl wnpgrem(BWINDOW *);


		/* Building borders.				    */

int cdecl wnputbor(DIM *,BORDER *,WHERE *);


	       /* Handling inter-window effects:  displaying,	    */
	       /* overlapping, removing.			    */

BWINDOW *cdecl wnneedup (BWINDOW *);
BWINDOW *cdecl wnnupblk (BWINDOW *,int,int,int,int);
BWINDOW *cdecl wnupblk (BWINDOW *,int, int,int, int);
WIN_NODE *cdecl wncover(WIN_NODE *,LOC *,DIM *);
int cdecl wnovrlap(LOC *,DIM *,LOC *,DIM *);
BWINDOW *cdecl wnforget(BWINDOW *);
BWINDOW *cdecl wnhide(BWINDOW *);
BWINDOW *cdecl wnunhide(BWINDOW *);


		/* Window output.				    */

void cdecl wnwrttyx (BWINDOW *, char, int, int, int);



#define DEF_BWINDOW  1		      /* Prevent second reading of    */
				      /* these definitions.	      */

#endif				      /* Ends "#ifndef DEF_BWINDOW"   */
