/**
*
* Name		GRLINE -- Draw a colored line.
*
* Synopsis	npts = grline(pstart,pend,color);
*
*		int npts	Returns number of points plotted.
*		PT *pstart	Pointer to starting point of line.
*		PT *pend	Pointer to ending point of line.
*		int color	Color of the line; chosen from the
*				currently set palette.
*
* Description	This function draws a line on the current display page
*		from *pstart to *pend using the specified color.  The
*		actual number of points plotted to draw the line is
*		returned.  If *pstart and *pend represent the same
*		location, only one point is plotted.
*
* Method	The variables x and y keep count of when the tracing
*		point coordinates whould be incremented.  The variable
*		fplot is used as a flag when a new point is plotted.
*		The same point is not plotted twice, because a color
*		value greater than 128 should XOR the current point.
*
* Returns	npts	Number of points plotted to trace the line.
*
* Version	5.00 (C)Copyright Blaise Computing Inc.  1983, 1987
*
**/


#include <bgraphs.h>
#include <butil.h>

#define GRDELTAX   (pend->x - pstart->x)
#define GRDELTAY   (pend->y - pstart->y)


int grline(pstart,pend,color)
PT  *pstart,*pend;
int color;
{
    int sdeltax        = utsign (GRDELTAX);
    int sdeltay        = utsign (GRDELTAY);
    int incx	       = utabs	(GRDELTAX);
    int incy	       = utabs	(GRDELTAY);
    int steps	       = utmax	(incx, incy);
    register int xerr  = 0;
    register int yerr  = 0;
    register int fplot = 0;
    int npts	       = 1;
    int i;
    PT	pos;

    /* Initialize the position variable, plot the first point and     */
    /* trace the rest of the line.				      */

    pos.x = pstart->x;
    pos.y = pstart->y;

    grptwrit (&pos,color);

    for (i = 0;  i <= steps;  i ++)
    {
	xerr += incx;
	yerr += incy;

	if (yerr > steps)
	{
	    yerr  -= steps;
	    pos.y += sdeltay;
	    fplot ++;
	}

	if (xerr > steps)
	{
	    xerr  -= steps;
	    pos.x += sdeltax;
	    fplot ++;
	}

	if (fplot)	/* Plot only if a new point.		    */
	{
	    grptwrit(&pos,color);
	    npts  ++;
	    fplot --;
	}
    }

    return (npts);
}
