/**
*
* Name		MNATR -- Change attributes on item in a menu.
*
* Synopsis	presult = mnatr (pmenu, pitem, attr);
*
*		BMENU *presult	Pointer to newly-changed BMENU
*				structure, or NIL if failure.
*		BITEM *pitem	Item to change attributes on.
*		int    attr	Attribute to set item to.
*
* Description	This function changes the attributes on a given item
*		in a given menu.  Nothing else in the menu's window
*		is changed.
*
* Returns	presult 	  Pointer to newly-changed BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  MN_BAD_MENU	   Pmenu is invalid.
*				  MN_BAD_ITEM	   Pitem is invalid.
*				  WN_BAD_WIN	   Pmenu->pwin is invalid.
*				  WN_NOT_SHOWN	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bmenu.h>


BMENU *mnatr (pmenu, pitem, attr)
BMENU *pmenu;
BITEM *pitem;
int    attr;
{
		/* Validate menu data structures.		    */
    mnvalidm (pmenu)

		/* Check validity of item.			    */
    if (pitem == NIL)
	wnreterr (WN_ILL_VALUE);
    if (pitem->signature != MN_ITEM_SIGN)
	wnreterr (MN_BAD_ITEM);

		/* Set attributes on the menu item.		    */
    if (wnatrblk (pmenu->pwin,
		  pitem->row, pitem->col,
		  pitem->row,
		  (pitem->col + pitem->len) - 1,
		  utlonyb (attr),
		  uthinyb (attr), WN_UPDATE) == NIL)
	return (NIL);

    return (pmenu);
}
