/**
*
* Name		MNDSTROY -- Detach menu from screen location
*			    and discard its memory structures.
*
* Synopsis	ret = mndstroy (pmenu);
*
*		int    ret	 Error code, or WN_NO_ERROR if ok.
*		BMENU *pmenu	 Pointer to BMENU structure to
*				 discard.
*
* Description	This function completely discards a Blaise C TOOLS menu
*		structure.  It undisplays the menu if necessary.
*
*		Do NOT use a menu structure (or any of its components)
*		after destroying it via MNDSTROY.  Its memory has been
*		freed.
*
*		An error occurs if pmenu does not point to a valid menu
*		structure.
*
* Returns	ret		  Error code, or WN_NO_ERROR if ok.
*		*pmenu		  Several fields altered.
*		b_pactnode[][]	  Window node with active cursor.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  MN_BAD_MENU	   *pmenu is invalid.
*				  MN_BAD_WIN	   Internal error.
*				  MN_BAD_KEY	   Bad keymap entry found
*				  MN_BAD_ITEM	   Bad menu item found.
*				  WN_NOT_SHOWN	   Internal error.
*				  WN_BAD_NODE	   Internal error.
*				  WN_BAD_PAGE	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bmenu.h>

#include <string.h>


int    mndstroy (pmenu)
BMENU *pmenu;
{
    int code;

		/* Validate menu structure.			    */
    if (mnvalmnu (pmenu) == NIL)
	wnretern (MN_BAD_MENU);

		/* Validate menu's window.                          */
    if (wnvalwin (pmenu->pwin) == NIL)
	wnretern (WN_BAD_WIN);

		/* If shown, undisplay window.			    */
    if (pmenu->pwin->where_shown.dev == COLOR ||
	pmenu->pwin->where_shown.dev == MONO)

	if (wnremove (pmenu->pwin) == NIL)
	    return (b_wnerr);

		/* Destroy menu's window data structures.           */
    if (wndstroy (pmenu->pwin))
	return (b_wnerr);

		/* Free memory used by item list.		    */
    if ((code = mndlitms (pmenu)) == WN_NO_ERROR)
		/* Free memory used by key list.		    */
	code = mndlkeys (pmenu);

		/* Invalidate menu signature.			    */
    strcpy (pmenu->signature, "Destroyed");

    return (code);
}
