/**
*
* Name		MNLITEM -- Add, change, or delete an item in a
*			   Lotus-style menu.
*
* Synopsis	presult = mnlitem (pmenu, row, col, option,
*				   pstring, lrow, lcol, plstring);
*
*		BMENU *presult	Pointer to menu which item and key(s)
*				were just added to, or NIL for failure.
*		BMENU *pmenu	Pointer to menu to which to add item.
*		int    row,	Row and column (relative to menu's
*		       col	window data area) at which to place
*				the item.
*		int    option	The bitwise OR-ing of the following values:
*				    MN_PROTECT
*					Item is protected.
*				    MN_NOPROTECT
*					Item is not protected.
*		char  *pstring	Text of the item.
*		int    lrow,	Row and column (relative to menu's
*		       lcol	window data area) at which to place
*				the item's description.
*		char  *plstring Text of the long description of the item.
*
* Description	This function writes a Lotus-style item to a menu.
*		An example is probably best here:
*
*		The call:
*		    presult = mnlitem (pmenu, 5, 2, MN_NOPROTECT, "Close",
*				       7, 1, "Close the currently-open file");
*
*		Will add the text "Close" to the menu at (5,2).  The
*		item will be in the normal-text attribute, because
*		"option" is MN_NOPROTECT.  It will also add the text
*		"Close the currently-open file" to the item data
*		structure.  If the menu is read with MNLREAD, when the
*		highlight bar is on "Close," the long description text
*		will appear at (7,1) in the longattr attribute given at
*		MNCREATE time.
*
*		An error occurs if row or col or lrow or lcol exceeds
*		the menu's window dimensions.
*
* Returns	presult 		Pointer to changed menu, or
*					NIL if failure.
*		b_wnerr 	Possible values:
*				(No change)	Success.
*				MN_BAD_MENU	Pmenu is invalid.
*				MN_BAD_ITEM	An item in pmenu is bad.
*				WN_BAD_WIN	Pmenu's window is invalid.
*				WN_NO_MEMORY	Insufficient memory.
*				WN_ILL_DIM	Row or col out of range
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <string.h>

#include <bmenu.h>


BMENU *mnlitem (pmenu, row, col, option, pstring, lrow, lcol, plstring)
BMENU *pmenu;
int    row, col, option;
char  *pstring;
int    lrow, lcol;
char  *plstring;
{
    BITEM *pitem;
    int    code;

		/* Add the standard text item to the menu.	    */
    if (NIL == mnitem (pmenu, row, col, option, pstring))
	return (NIL);

		/* Get a pointer to the item so we can add special  */
		/* "Lotus"-style information; if we cannot get a    */
		/* pointer, the above MNITEM call was a deletion.   */
    if ((pitem = mnmchitm (pmenu, NIL, row, col, 1, &code)) != NIL)
    {
		/* Add the information. 			    */
	pitem->lrow	= lrow;
	pitem->lcol	= lcol;
	pitem->plstring = plstring;
	pitem->llen	= mntrunc (pmenu, plstring, lcol);
    }

    if (code != WN_NO_ERROR)
	return (NIL);
    else
	return (pmenu);
}
