/**
*
* Name		PRCANCEL -- Cancel a file (or all files) from DOS
*			    print spooler.
*
* Synopsis	ret = prcancel (pfile);
*
*		int ret 	Return code -- 0 (PR_OK) if no error.
*
*		char *pfile	Filename to cancel from spool queue.
*
* Description	This function cancels the specified filename from the
*		DOS print spooler queue.  This is the same as typing
*
*			PRINT /c file
*
*		on the DOS command line.
*
*		If pfile is NIL or the string pointed to by pfile has
*		length zero, all files are cancelled from the queue.
*		This is equivalent to typing:
*
*			PRINT /t
*
*		The DOS filename expansion characters ('*' and '?') are
*		allowed as characters in the filename and act as usual.
*
* Special	If the DOS the machine is running is older than 3.00,
* Cases 	then PRCANCEL always returns PR_INSTAL (1).
*
* Returns	ret		    0 (PR_OK) if no error, otherwise
*				    one of the following values:
*
*		    0 (PR_OK)	    File(s) successfully removed from
*				    spooler queue.
*
*		    1 (PR_INSTAL)   Spooler not installed (or)
*				    MS/PC-DOS older than 3.00.
*
*		    9 (PR_BUSY)     Spooler busy, unable to get
*				    attention.	Try again later.
*
* Version	5.00  (C)Copyright Blaise Computing Inc. 1987
*
**/

#include <dos.h>

#include <bprint.h>
#include <butil.h>

#define NUL '\0'

int prcancel (pfile)
char *pfile;
{
    struct SREGS sregs;
    union  REGS  regs;

		/* If print spooler not installed, exit.	    */
    if (! prinstld ())
	return (PR_INSTAL);

		/* If path is NIL or of length 0, set up to cancel  */
		/* up to cancel all files in queue.		    */
    if ((pfile == NIL) || (pfile[0] == NUL))
	regs.x.ax = 0x0103;

    else
    {		/* When we put 0x0102 in AX, we are telling the     */
		/* request processor that we mean device 1 (print   */
		/* spooler), request 2 (cancel file).		    */
	regs.x.ax = 0x0102;

		/* Point to the pathname of the file(s) to cancel   */
		/* from the queue.				    */
	sregs.ds  = utseg ((char far *) pfile);
	regs.x.dx = utoff ((char far *) pfile);
    }

		/* Do the request.				    */
    int86x (PR_DOS_INT, &regs, &regs, &sregs);

		/* If there was an error, return it.		    */
    if (regs.x.cflag)
	return (regs.x.ax);

    return (PR_OK);
}
