/**
*
* Name		SCMODE4 -- Set mode 4 palette and background color.
*
* Synopsis	ret = scmode4 (palette, backgrd);
*
*		int ret 	-1 if error, 0 if successful.
*		int palette	Foreground palette for mode 4:
*				    0 - green/red/yellow.
*				    1 - cyan/magenta/white.
*		int backgrd	Background color (0-31).
*
* Description	This function sets the dot palette and the background
*		color for video mode 4 (320x200 4-color graphics).
*
*		An error occurs if the current video adapter is not in
*		mode 4.
*
* Returns	ret		-1 if error, otherwise 0.
*
* Version	5.00 (C)Copyright Blaise Computing Inc.  1987
*
**/


#include <dos.h>

#include <bgraphs.h>
#include <bscreens.h>


int scmode4 (palette, backgrd)
int palette, backgrd;
{
    union REGS regs;		/* Registers for int86() call.	    */
    int mode;			/* Video mode.			    */
    int cols;			/* Number of columns on screen.     */
    int apage;			/* Active display page. 	    */

    scmode (&mode, &cols, &apage);

    if (mode == 4)
    {
	regs.h.ah = 0x0b;	/* Go do interrupt to set palette.  */
	regs.h.bh = 1;
	regs.h.bl = (unsigned char) palette;
	int86 (GR_BIOS_INT, &regs, &regs);

	regs.h.ah = 0x0b;			/* Go do interrupt  */
	regs.h.bh = 0;				/* to set background*/
	regs.h.bl = (unsigned char) backgrd;	/* and intensity.   */
	int86 (GR_BIOS_INT, &regs, &regs);

	return (0);
    }

    return (-1);		/* Mode not equal to 4. 	    */
}
