/**
*
* Name		UTMOVMEM -- Copy memory from a far pointer to another.
*
* Synopsis	utmovmem (source, target, amount);
*
*		char far *source	A far pointer to the bytes
*					to be moved.
*
*		char far *target	A far pointer to the place the
*					bytes are to be moved to.
*
*		unsigned int amount	The number of bytes to move.
*
* Description	UTMOVMEM copies memory from a source region to a target
*		region.  It is a smart memory move routine, meaning that
*		it can be used for overlapping memory moves.
*
*		Cases that UTMOVMEM can deal with are:
*
*
*	    1.	SourceSourceSource
*					TargetTargetTarget
*
*	    2.		SourceSourceSource
*				TargetTargetTarget
*
*	    3.		TargetTargetTarget
*				SourceSourceSource
*
*	    4.	TargetTargetTarget
*					SourceSourceSource
*
* Returns	Nothing.
*
* Version	5.00  (C)Copyright Blaise Computing Inc.  1987
*
**/


#include <butil.h>


void utmovmem (source, target, num)
char far *source;
char far *target;
register unsigned int num;
{
    char far *snum;
    char far *tnum;


    /* Normalize source and target pointers so we can compare them. */
    source = utnorm (source, char);
    target = utnorm (target, char);

    /* Adjust to end of source and target regions, and normalize.   */
    snum = utnorm ((source + num), char);
    tnum = utnorm ((target + num), char);

    if ((num != 0) && (source != target))
    {
	if (
			    /* Source completely before target.     */
	    (snum <= target) ||
			    /* Source completely after target.	    */
	    (source >= tnum) ||
			    /* Target covers source left-to-right.  */
	    ((source > target) &&
		(source <= tnum)))

	    utamove (source, target, num, UT_MVFORWARD);


	else
	{		    /* Source covers target left-to-right.  */
	    target += num - 1;
	    source += num - 1;

	    utamove (source, target, num, UT_MVBACKWARD);
	}
    }
}
