/**
*
* Name		WNATRSTR -- Change attributes on stream in a window.
*
* Synopsis	presult = wnatrstr (pwin, row, col, count,
*				    fore, back, option);
*
*		BWINDOW *presult  Pointer to newly-changed BWINDOW
*				  structure, or NIL if failure.
*		BWINDOW *pwin	  Pointer to BWINDOW structure
*				  to change.
*		int row, col	  Row and column (relative to upper
*				  left corner of window) of start of
*				  stream to set.
*		int count	  Number of window character positions
*				  to set.
*		int fore	  New foreground attribute (-1 if
*				  foreground to be unchanged).
*		int back	  New background attribute (-1 if
*				  background to be unchanged).
*		int option	  WN_UPDATE to update the window (on the
*				  screen) after changing attributes;
*				  WN_NO_UPDATE to not update -- the
*				  attribute change will be in memory only.
*
* Description	This function fills a stream of the data area of a
*		window with a given attribute without affecting the
*		characters already displayed there.  The cursor is
*		not moved.
*
*		If fore or back is -1, then foreground or background
*		attributes are left unchanged, respectively.
*
*		This function does not affect the window's default
*		attributes.  Use WNSETOPT for that purpose.
*
* Returns	presult 	  Pointer to newly-changed BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   pwin is invalid.
*				  WN_NOT_SHOWN	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_ILL_DIM	   Bad row and/or column.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bwindow.h>

#define H     (pwin->img.dim.h)
#define W     (pwin->img.dim.w)
#define AREA  (H * W)
#define START ((W * row) + col)


BWINDOW *wnatrstr (pwin, row, col, count, fore, back, option)
BWINDOW *pwin;
int	 row, col;
int	 count;
int	 fore, back;
int	 option;
{
    int   endpos, i;
    CELL *pdata;
    char  oldmask = '\0';
    char  attr;

		/* Validate window data structure.		    */
    wnvalidw (pwin)

		/* Make sure (row,col) is in window.		    */
    if (utrange (row, 0, H - 1) ||
	utrange (col, 0, W - 1))
	wnreterr (WN_ILL_DIM);

		/* "oldmask" will help us extract the portions of  */
		/* the existing attributes that we want to keep.   */
    if (fore == -1)
	oldmask  = 0x0f;
    if (back == -1)
	oldmask |= 0xf0;

    attr   = (char) (utnybbyt (back, fore) & ~oldmask);
    endpos = ((START + count > AREA) ? (AREA - START) : (count)) + START;
    pdata  = pwin->img.pdata;

    for (i = START;  i < endpos;  i ++)
	pdata[i].attr = (pdata[i].attr & oldmask) | attr;

		/* Write whole window unless delayed or not shown.  */
    if ( !(option & WN_NO_UPDATE))
    {
		/* Update just a section of this line if we can get */
		/* away with it.				    */
	if (row == (int) endpos / W)
	    wnnupblk (pwin, row, col, row, col + (i - START) - 1);

	else	/* Otherwise update all lines changed.		    */
	    wnnupblk (pwin, row, 0, (int) endpos / W, W - 1);
    }

    pwin->internals.dirty = 1;
    return (pwin);
}
