/**
*
* Name		WNATTR -- Change attributes on current window.
*
* Synopsis	presult = wnattr (fore, back);
*
*		BWINDOW *presult  Pointer to newly-changed BWINDOW
*				  structure, or NIL if failure.
*		int fore	  New foreground attribute (-1 if
*				  foreground to be unchanged).
*		int back	  New background attribute (-1 if
*				  background to be unchanged).
*
* Description	This function fills the data area of the current window
*		with a given attribute without affecting the characters
*		already displayed there.  The cursor is not moved.
*
*		If fore or back is -1, then foreground or background
*		attributes are left unchanged, respectively.
*
*		This function does not affect the window's default
*		attributes.  Use WNSETOPT for that purpose.
*
*		An error occurs if no window is current.
*
* Returns	presult 	  Pointer to newly-changed BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   b_pcurwin is invalid.
*				  WN_NOT_SHOWN	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_ILL_DIM	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


BWINDOW *wnattr (fore, back)
int fore, back;
{
		/* Validate current window's data structures.       */
    wnvalidw (b_pcurwin)

		/* Write the whole window with the given attribute. */
    return (wnatrstr (b_pcurwin, 0, 0,
		      b_pcurwin->img.dim.h * b_pcurwin->img.dim.w,
		      fore, back, WN_UPDATE));
}
