/**
*
* Name		WNCOVER -- Mark lower windows if they are covered by
*			   a given rectangular region.
*
* Synopsis	presult = wncover (pnode, pcorner, pdim);
*
*		WIN_NODE *presult Pointer to uppermost window node
*				  examined, or NIL if failure.
*		WIN_NODE *pnode   Pointer to uppermost window node to
*				  examine.
*		LOC *pcorner	  Pointer to structure denoting upper
*				  left corner of rectangular region.
*		DIM *pdim	  Pointer to structure denoting
*				  dimensions of rectangular region.
*
* Description	This function compares all windows below a given window
*		against a given rectangular region and marks those
*		windows as covered that are overlapped by the region.
*
*		If the window with the active cursor is covered by the
*		rectangle, then its cursor is deactivated and the
*		screen's visible cursor is turned off.
*
* Returns	presult 	  Pointer to uppermost window node
*				  examined, or NIL if failure.
*		b_pactnode[][]	  Window node with active cursor.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_NODE	   Invalid pnode.
*				  WN_BAD_WIN	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


WIN_NODE *wncover (pnode, pcorner, pdim)
WIN_NODE *pnode;
LOC	 *pcorner;
DIM	 *pdim;
{
    WIN_NODE *presult;
    BWINDOW  *pwin;
    int      row, col, high, low;

    wnvalidn (pnode)
    pwin = pnode->pwin;
    wnvalidw (pwin)

    if (wnovrlap (pcorner,	/* If rectangle overlaps this	    */
		  pdim, 	/* window,			    */
		  &(pwin->where_shown.corner),
		  &(pwin->img.dim)))
    {				/* mark this window as covered.     */
	pwin->internals.frozen		 =
	pwin->internals.any_data_covered =
	pwin->internals.cur_frozen	 = 1;

				/* If this window's cursor is       */
				/* activated, turn physical cursor  */
				/* off and deactivate window.	    */
	if (pnode == b_pactnode[pwin->where_shown.dev][pwin->where_shown.page])
	{
	    sccurst (&row, &col, &high, &low);
	    scpgcur (1, high, low, CUR_NO_ADJUST);
	    b_pactnode[pwin->where_shown.dev][pwin->where_shown.page] = NIL;
	}
    }

    presult = pnode;
    if (pnode->below != NIL)	/* Continue with lower windows.     */
	if (wncover (pnode->below, pcorner, pdim) == NIL)
	    presult = NIL;

    return (presult);
}
