/**
*
* Name		WNCREATE -- Allocate and create a window structure.
*
* Synopsis	presult = wncreate (height, width, attr);
*
*		BWINDOW *presult  Pointer to newly-created BWINDOW
*				  structure, or NIL if failure.
*		int height	  Number of rows    in data area.
*		int width	  Number of columns in data area.
*		int attr	  Default attribute in data area.
*
* Description	This function allocates space for a window structure and
*		fills it with default values.  This includes allocating
*		and initializing the window's data area with blanks
*		(' ') having the specified attribute.
*
*		An error occurs if the window dimensions exceed maximum
*		screen dimensions or if the memory allocation fails.
*
*		Use WNDSTROY to discard the BWINDOW structure and
*		related internal data structures.
*
* Returns	presult 	  Pointer to newly-created BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	   Possible values:
*				  (No change)	  Success.
*				  WN_ILL_DIM	  Invalid dimensions.
*				  WN_NO_MEMORY	  Insufficient memory.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <stdlib.h>
#include <string.h>

#include <bwindow.h>


int b_wnerr = WN_NO_ERROR;	/* Most recent error (WN_NO_ERROR   */
				/* if none).			    */


BWINDOW *wncreate(height,width,attr)
int height,width,attr;
{
    BWINDOW *pwin;
    CELL    *ptr;
    int     area,i;

		/* Validate size.				    */
    if (utrange (height, 1, PC_BIG_ROWS) ||
	utrange (width,  1, PC_COLS))
	wnreterr (WN_ILL_DIM);

		/* Allocate space for window structure. 	    */
    if ((pwin = utalloc (BWINDOW)) == NIL)
	wnreterr (WN_NO_MEMORY);

    pwin->img.dim.h = height;
    pwin->img.dim.w = width;

		/* Allocate space for node structure.		    */
    if ((pwin->pnode = utalloc (WIN_NODE)) == NIL)
    {
	free ((char *) pwin);
	wnreterr (WN_NO_MEMORY);
    }

    pwin->pnode->pwin = pwin;

		/* Allocate space for memory image.		    */
    if ((pwin->img.pdata =
	 (CELL *) calloc ((unsigned int)
			  (pwin->img.dim.h * pwin->img.dim.w),
			   sizeof (CELL))) == NIL)
    {
	free ((char *) pwin->pnode);
	free ((char *) pwin);
	wnreterr (WN_NO_MEMORY);
    }

		/* Initialize window to all blanks with standard    */
		/* attribute for this window.			    */
    area = height * width;
    for (ptr = pwin->img.pdata, i = 0;	i < area;  i++, ptr++)
    {
	ptr->ch   = ' ';
	ptr->attr = (char) attr;
    }

		/* Initialize fields to default values (except	    */
		/* where 0 is an acceptable default value).	    */
    pwin->attr		    = attr;
    pwin->where_shown.dev   = ABSENT;
    pwin->cur_type.high     = 12;
    pwin->cur_type.low	    = 13;
    pwin->options.removable = 1;

		/* Distinctive signatures.			    */
    strcpy (pwin->signature, BWIN_SIGN);
    strcpy (pwin->pnode->signature, BNODE_SIGN);

    return (pwin);
}
