/**
*
* Name		WNCURMOV -- Move the current window's cursor.
*
* Synopsis	presult = wncurmov (row, col);
*
*		BWINDOW *presult  Pointer to current window structure,
*				  or NIL if failure.
*		int row,col	  Row and column (relative to upper left
*				  corner of window) of new cursor
*				  position.
*
* Description	The current window's cursor is moved.  No change is
*		visible unless the window has been selected via WNCURSOR
*		to have the active cursor on its display page.
*
*		An error occurs if the position is beyond the bounds of
*		the window.
*
* Returns	presult 	  Pointer to current BWINDOW
*				  structure, or NIL if failure.
*		b_pcurwin->cur_loc Current BWINDOW cursor position,
*				  or unchanged if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_ILL_DIM	   Position out of range.
*				  WN_BAD_WIN	   No window designated
*						   as current.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


BWINDOW *wncurmov (row, col)
int row, col;
{
    wnvalidw (b_pcurwin)

		/* Validate requested position. 		    */
    if (utrange (row, 0, b_pcurwin->img.dim.h - 1) ||
	utrange (col, 0, b_pcurwin->img.dim.w - 1))
	wnreterr (WN_ILL_DIM);

		/* Assign row and column.			    */
    b_pcurwin->cur_loc.row = row;
    b_pcurwin->cur_loc.col = col;

		/* If the window is displayed and not frozen, move  */
		/* The physical cursor. 			    */
    if ((b_pcurwin->where_shown.dev != ABSENT) &&
	(!b_pcurwin->internals.cur_frozen)     &&
	(!b_pcurwin->options.delayed))
	sccurset (b_pcurwin->where_shown.corner.row + row,
		  b_pcurwin->where_shown.corner.col + col);

    return (b_pcurwin);
}
