/**
*
* Name		WNCURSOR -- Activate a window's cursor and deactivate
*			    the cursors of all other windows showing on
*			    that display page.
*
* Synopsis	presult = wncursor (pwindow);
*
*		BWINDOW *presult  Pointer to newly-selected BWINDOW
*				  structure, or NIL if failure.
*		BWINDOW *pwindow  Pointer to BWINDOW structure to
*				  select.
*
* Description	The window's cursor is activated (although the cursor
*		may be invisible).  The cursors of other windows on that
*		page are deactivated.
*
*		An error occurs if pwindow does not point to a valid
*		window structure, if the window is hidden or not
*		currently displayed, or if some portion of the window's
*		data area is covered by another window.
*
* Returns	presult 	  Pointer to newly-selected BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   *pwindow is invalid.
*				  WN_NOT_SHOWN	   Hidden or not
*						   currently shown.
*				  WN_COVERED	   Data area partly
*						   covered.
*				  WN_BAD_NODE	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>

			/* The window (on each page) selected to    */
			/* have a visible cursor.		    */
WIN_NODE *(b_pactnode[MAX_DEVICES][MAX_PAGES]) = {0};


BWINDOW *wncursor (pwindow)
BWINDOW *pwindow;
{
    int old_dev, old_page, old_npage;
    int mode, columns, act_page;

		/* Validate window and window's node data structures*/
    wnvalidw (pwindow)
    wnvalidn (pwindow->pnode)

    if (pwindow->internals.any_data_covered)
	wnreterr (WN_COVERED);

    if (pwindow->options.hidden)
	wnreterr (WN_NOT_SHOWN);

		/* Note former device; note current page on former  */
		/* device; select and validate device and page.     */
    old_dev  = scmode (&mode, &columns, &act_page);
    old_page = b_curpage;
    if (wnseldev (&pwindow->where_shown, &pwindow->img.dim, &old_npage))
	wnreterr (WN_NOT_SHOWN);

		/* If a window on this page is currently designated */
		/* to control the cursor, deactivate it.	    */
    if (b_pactnode[pwindow->where_shown.dev][pwindow->where_shown.page]
	!= NIL)
	b_pactnode[pwindow->where_shown.dev][pwindow->where_shown.page]
	    ->pwin->internals.cur_frozen = 1;

		/* Activate this window instead.		    */
    b_pactnode[pwindow->where_shown.dev][pwindow->where_shown.page] =
	pwindow->pnode;

    pwindow->internals.cur_frozen = 0;

		/* Move the cursor.				    */
    sccurset (pwindow->where_shown.corner.row + pwindow->cur_loc.row,
	      pwindow->where_shown.corner.col + pwindow->cur_loc.col);

		/* Set cursor type.				    */
    scpgcur (pwindow->options.cur_off,
	     pwindow->cur_type.high,
	     pwindow->cur_type.low,
	     CUR_ADJUST);

		/* Restore current page on new device; restore old  */
		/* device; restore current page on old device.	    */
    scpage (old_npage);
    scchgdev (old_dev);
    scpage (old_page);

    return (pwindow);
}
