/**
*
* Name		WNDSTROY -- Detach window from screen location
*			    and discard its memory structures.
*
* Synopsis	result = wndstroy (pwin);
*
*		int	result	  Error code, or WN_NO_ERROR if ok.
*		BWINDOW *pwin	  Pointer to BWINDOW structure to
*				  discard.
*
* Description	This function completely discards a Blaise C TOOLS window
*		structure.  It does not affect any video display page.
*
*		Do NOT use a window structure (or any of its components)
*		after destroying it via WNDSTROY.  Its memory has been
*		freed.
*
*		An error occurs if pwin does not point to a valid window
*		structure.
*
*		Use WNREMOVE to clean up the screen gracefully before
*		destroying the window with WNDSTROY.  Use WNFORGET to
*		abandon a window's border and location without
*		destroying the window itself.  Use WNHIDE to make the
*		window temporarily invisible.
*
* Returns	result		  Error code, or WN_NO_ERROR if ok.
*		*pwin		  Several fields altered.
*		b_pactnode[][]	  Window node with active cursor.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   *pwin is invalid.
*				  WN_NOT_SHOWN	   Internal error.
*				  WN_BAD_NODE	   Internal error.
*				  WN_BAD_PAGE	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <string.h>

#include <bwindow.h>


int wndstroy (pwin)
BWINDOW *pwin;
{
		/* Validate window pointer.			    */
    if (wnvalwin (pwin) == NIL)
	wnretern (WN_BAD_WIN);

		/* If it's showing, abandon border & location.      */
    if (pwin->where_shown.dev == MONO ||
	pwin->where_shown.dev == COLOR)
	if (NIL == wnforget (pwin))
	    return (b_wnerr);

		/* Free the image of previous screen data.	    */
    if (pwin->prev.pdata != NIL)
    {
	free ((char *) pwin->prev.pdata);
	pwin->prev.pdata = NIL;
    }

		/* Free the memory copy of the window's data area.  */
    if (pwin->img.pdata != NIL)
    {
	free ((char *) pwin->img.pdata);
	pwin->img.pdata = NIL;
    }

		/* Mark the node abandoned.			    */
    strcpy (pwin->pnode->signature,"Closed");

		/* Free the node.				    */
    pwin->pnode->pwin = NIL;
    free ((char *) pwin->pnode);
    pwin->pnode = NIL;

		/* Mark the window abandoned.			    */
    strcpy (pwin->signature,"Destroyed");

		/* Free the window itself.			    */
    free ((char *) pwin);

    return (WN_NO_ERROR);
}
