/**
*
* Name		WNFORGET -- Detach window from screen location
*			    without cleaning screen up.
*
* Synopsis	presult = wnforget (pwin);
*
*		BWINDOW *presult  Pointer to newly-removed BWINDOW
*				  structure, or NIL if failure.
*		BWINDOW *pwin	  Pointer to BWINDOW structure to
*				  remove.
*
* Description	This function "removes" a window from the video display
*		page where it is displayed without actually modifying
*		any displayed characters.  That is, the previous screen
*		data is not replaced (even in the case of a removable
*		window).
*
*		An error occurs if pwin does not point to a valid window
*		structure or if the window is not currently displayed.
*
*		Use WNREMOVE to gracefully clean up the screen as the
*		window is detached from it.  Use WNDSTROY (with or
*		without WNFORGET) to discard the entire window structure
*		as well.
*
* Returns	presult 	  Pointer to newly-removed BWINDOW
*				  structure, or NIL if failure.
*		*pwin		  Several fields altered.
*		b_pactnode[][]	  Window node with active cursor.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   *pwin is invalid.
*				  WN_NOT_SHOWN	   Not currently shown.
*				  WN_BAD_NODE	   Internal error.
*				  WN_BAD_PAGE	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


BWINDOW *wnforget (pwin)
BWINDOW *pwin;
{
    BWINDOW *presult;

		/* Validate window pointer.			    */
    wnvalidw (pwin)

		/* Ensure it's showing.                             */
    if (pwin->where_shown.dev != MONO &&
	pwin->where_shown.dev != COLOR)
	wnreterr (WN_NOT_SHOWN);

		/* If window is current, mark no window current.    */
    if (b_pcurwin == pwin)
	b_pcurwin = NIL;

    if (b_pactnode[pwin->where_shown.dev][pwin->where_shown.page]
	->pwin == pwin)
		/* Then this window is the one with active cursor.  */
	b_pactnode[pwin->where_shown.dev][pwin->where_shown.page] = NIL;

		/* Now no window has an active cursor.	(But don't  */
		/* touch the visible cursor.)			    */
    pwin->internals.cur_frozen = 1;

		/* Remove from linked list.			    */
    if ((presult = wnpgrem (pwin))  != NIL)
    {
	pwin->where_shown.dev =
	pwin->where_prev.dev  = ABSENT;
	pwin->internals.dirty =
	pwin->options.hidden  = 0;
    }

		/* Correct internal flag in case it remains set.    */
    pwin->internals.temp_hid = 0;

    return (presult);
}
