/**
*
* Name		WNHORIZ -- Horizontally scroll the current window.
*
* Synopsis	presult = wnhoriz (num_cols, fore, back, dir);
*
*		BWINDOW *presult  Pointer to newly-scrolled BWINDOW
*				  structure, or NIL if failure.
*		int num_cols	  Number of columns to scroll.	A value
*				  of 0 clears the window.
*		int fore,back	  Foreground and background attributes of
*				  new blank columns.  (-1 specifies
*				  window's native attributes.)
*		int dir 	  Scrolling direction (SCR_LEFT or
*				  SCR_RIGHT).
*
* Description	This function moves columns of characters (with their
*		attributes) to the left or right within the current
*		window.  The vacant columns are filled with blanks and a
*		specified attribute.
*
*		An error occurs if no window is current.
*
* Returns	presult 	  Pointer to newly-scrolled BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   b_pcurwin is invalid.
*				  WN_NOT_SHOWN	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_ILL_DIM	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>

#define H (b_pcurwin->img.dim.h)
#define W (b_pcurwin->img.dim.w)


BWINDOW *wnhoriz (num_cols, fore, back, dir)
int	 num_cols, fore, back, dir;
{
		/* Validate window data structures.		    */
    wnvalidw (b_pcurwin)

		/* Force num_cols to be 0 (clear window) if not in  */
		/* range.					    */
    if (utrange (num_cols, 1, H))
	num_cols = 0;

    if (dir == SCR_LEFT)
	return (wnscrblk (b_pcurwin,
			  0,	  0,
			  H - 1,  W - 1,
			  fore, back, WNSCR_LEFT, num_cols, WN_UPDATE));

    return (wnscrblk (b_pcurwin,
		      0,      0,
		      H - 1,  W - 1,
		      fore, back, WNSCR_RIGHT, num_cols, WN_UPDATE));
}
