/**
*
* Name		WNNEEDUP -- Mark a window as "dirty," and do update if
*			    possible.
*
* Synopsis	presult = wnneedup (pwin);
*
*		BWINDOW *presult  Pointer to BWINDOW structure updated,
*				  or NIL if failure.
*		BWINDOW *pwin	  Pointer to BWINDOW structure to
*				  update.
*
* Description	This function writes any pending I/O requests to the
*		designated window if there are any waiting, and the
*		window is not delayed, not frozen, etc.
*
*		No error checking is done!
*
* Returns	presult 	  Pointer to newly-hidden BWINDOW
*				  structure, or NIL if failure.
*		*pwin		  Several fields altered.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   *pwin is invalid.
*				  WN_BAD_DEV	   Internal error.
*				  WN_ILL_DIM	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bwindow.h>


BWINDOW *wnneedup (pwin)
BWINDOW *pwin;
{
    pwin->internals.dirty = 1;

    if ((pwin->where_shown.dev != MONO) &&
	(pwin->where_shown.dev != COLOR))
				/* Not shown anywhere.		    */
	pwin->where_shown.dev = ABSENT;

    return (pwin->options.delayed ||
	    pwin->where_shown.dev == ABSENT)
	    ? (pwin)
	    : (wnupdate (pwin));
}
