/**
*
* Name		WNNUPBLK -- Mark a window as "dirty," and do update on
*			    a section if possible.
*
* Synopsis	presult = wnnupblk (pwin, r1, c1, r2, c2);
*
*		BWINDOW *presult  Pointer to BWINDOW structure updated,
*				  or NIL if failure.
*		BWINDOW *pwin	  Pointer to BWINDOW structure to
*				  update.
*		int r1, c1	  Row and column of upper-left corner
*				  of block to attempt to update.
*				  (relative to upper-left corner of
*				  window).
*		int r2, c2	  Row and column of lower-right corner
*				  of block to attempt to update.
*
* Description	This function writes any pending I/O requests which
*		are in the rectangle bounded by (r1,c1)-(r2,c2) to the
*		designated window if there are any waiting, and the
*		window is not delayed, not frozen, etc.
*
* Returns	presult 	  Pointer to newly-hidden BWINDOW
*				  structure, or NIL if failure.
*		*pwin		  Several fields altered.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   *pwin is invalid.
*				  WN_ILL_DIM	   Bad coordinates.
*				  WN_BAD_DEV	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bwindow.h>


BWINDOW *wnnupblk (pwin, r1, c1, r2, c2)
BWINDOW *pwin;
int	 r1, c1, r2, c2;
{
    if ((pwin->where_shown.dev != MONO) &&
	(pwin->where_shown.dev != COLOR))
				/* Not shown anywhere.		    */
	pwin->where_shown.dev = ABSENT;

		/* If this window was already dirty, do a full	    */
		/* update.					    */
    if (pwin->internals.dirty == 1)
	return (pwin->options.delayed ||
		pwin->where_shown.dev == ABSENT)
		? (pwin)
		: (wnupdate (pwin));

		/* Otherwise update just a block.		    */
    pwin->internals.dirty = 1;

    return (pwin->options.delayed ||
	    pwin->where_shown.dev == ABSENT)
	    ? (pwin)
	    : (wnupblk (pwin, r1, c1, r2, c2));
}
