/**
*
* Name		WNOVRLAP -- Report whether two rectangular regions
*			    overlap.
*
* Synopsis	result = wnovrlap (pcorner1, pdim1, pcorner2, pdim2);
*
*		int  result	  1 if they overlap, 0 if not.
*
*		LOC *pcorner1,
*		    *pcorner2	  Pointers to structures designating
*				  upper left corners of the two regions.
*		DIM *pdim1,
*		    *pdim2	  Pointers to structures designating
*				  the dimensions of the two regions.
*
* Description	This function compares two rectangular regions (assumed
*		to lie on the same display screen) and reports whether
*		they overlap.
*
*		This function assumes that all its data is correct and
*		performs no data checking.
*
* Returns	result		  1 if they overlap, 0 if not.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>

#define  BOT_ROW1  (pcorner1->row + pdim1->h - 1)
#define  BOT_ROW2  (pcorner2->row + pdim2->h - 1)
#define  RT_COL1   (pcorner1->col + pdim1->w - 1)
#define  RT_COL2   (pcorner2->col + pdim2->w - 1)


int wnovrlap (pcorner1, pdim1, pcorner2, pdim2)
LOC *pcorner1;
DIM *pdim1;
LOC *pcorner2;
DIM *pdim2;
{
    return (BOT_ROW1 >= pcorner2->row &&
	    BOT_ROW2 >= pcorner1->row &&
	    RT_COL1  >= pcorner2->col &&
	    RT_COL2  >= pcorner1->col);
}
