/**
*
* Name		WNPGADD -- Add window to linked list of windows
*			   displayed on a device & display page.
*
* Synopsis	presult = wnpgadd (pwindow, dev, page);
*
*		WIN_NODE *presult   Pointer to newly-created WIN_NODE
*				    structure, or NIL if failure.
*		BWINDOW  *pwindow   Window to add to list.
*		int	  dev	    Video display device (COLOR or MONO).
*		int	  page	    Video display page.
*
* Description	This function adds a window to the linked list which
*		governs the windows displayed on a given device and
*		video page.  The window is stored as the topmost (i.e.,
*		most recently displayed).
*
*		An error occurs if the window structure is invalid, or
*		if the device or page are illegal.
*
* Returns	presult 	  Pointer to newly-created WIN_NODE
*				  structure, or NIL if failure.
*		b_wnlist[][]	  Altered linked list if success.
*		pwindow->pnode	  Pointer to newly-created WIN_NODE
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	  Success.
*				  WN_BAD_WIN	  *pwin is invalid.
*				  WN_BAD_DEV	  Unknown device.
*				  WN_BAD_PAGE	  Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>

		/* List of windows displayed on each page (NIL if   */
		/* none).					    */
WIN_NODE *(b_wnlist[MAX_DEVICES][MAX_PAGES]) = {0};


WIN_NODE *wnpgadd (pwindow, dev, page)
BWINDOW *pwindow;
int dev,page;
{
    wnvalidw (pwindow)
    wnvalidn (pwindow->pnode)

		/* Make sure dev & page are within range of array   */
		/* subscripts.					    */
    if (dev != MONO && dev != COLOR)
	wnreterr (WN_BAD_DEV);

    if (utrange (page, 0, MAX_PAGES - 1))
	wnreterr (WN_BAD_PAGE);

		/* Attach node to window.			    */
    pwindow->pnode->pwin = pwindow;

		/* Attach to former topmost window.		    */
    pwindow->pnode->below = b_wnlist[dev][page];

		/* Attach former topmost to this window.	    */
    if (pwindow->pnode->below != NIL)
	pwindow->pnode->below->above = pwindow->pnode;

		/* New head of list.				    */
    return (b_wnlist[dev][page] = pwindow->pnode);
}
