/**
*
* Name		WNPGREM -- Remove window from linked list of windows
*			   displayed on a device & display page.
*
* Synopsis	presult = wnpgrem (pwindow);
*
*		WIN_NODE *presult  Pointer to newly-removed BWINDOW
*				   structure, or NIL if failure.
*		BWINDOW  *pwindow  Window to remove from list.
*
* Description	This function removes a window from the linked list
*		which governs the windows displayed on its video device
*		and display page.
*
*		An error occurs if the window structure is invalid, if
*		the window is not attached to a location on a video
*		display device, or if the window is not attached to a
*		valid node.
*
* Returns	presult 	  Pointer to newly-removed BWINDOW
*				  structure, or NIL if failure.
*		b_wnlist[][]	  Altered linked list if success.
*		pwindow->pnode	  NIL if success.
*		b_wnerr 	  Possible values:
*				  (No change)	  Success.
*				  WN_BAD_WIN	  *pwindow is invalid.
*				  WN_BAD_NODE	  *pwindow not attached
*						  to valid node.
*				  WN_BAD_DEV	  Not shown.
*				  WN_BAD_PAGE	  Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


BWINDOW *wnpgrem (pwindow)
BWINDOW *pwindow;
{
    WIN_NODE *pnode;

    wnvalidw (pwindow)
    pnode = pwindow->pnode;
    wnvalidn (pnode)

		/* Make sure window is displayed.		    */
   if (pwindow->where_shown.dev != MONO &&
       pwindow->where_shown.dev != COLOR)
	wnreterr (WN_BAD_DEV);

		/* Make sure window's page is within reasonable     */
		/* limits.					    */
    if (utrange (pwindow->where_shown.page, 0, MAX_PAGES - 1))
	wnreterr (WN_BAD_PAGE);

		/* If this is top window, link root of list to next */
		/* window.					    */
    if (b_wnlist[pwindow->where_shown.dev][pwindow->where_shown.page]
	== pnode)
	b_wnlist[pwindow->where_shown.dev][pwindow->where_shown.page]
	= pnode->below;

		/* Link next higher window to next lower  window.   */
    if (pnode->above != NIL)
	pnode->above->below = pnode->below;

		/* Link next lower  window to next higher window.   */
    if (pnode->below != NIL)
	pnode->below->above = pnode->above;

    pnode->above   =			/* Detach from next higher  */
					/*     window.		    */
    pnode->below   = NIL;		/* Detach from next lower   */

    return (pwindow);
}
