/**
*
* Name		WNPRINTF -- Printf to the current window.
*
* Synopsis	ret = wnprintf (fmt [,arg[,arg]]);
*
*		int	 ret	  Less than zero if error, otherwise the
*				  number of character printed to the
*				  window.
*		char	*fmt	  The printf-specification of the arguments.
*		type	 arg	  The argument(s) specified in fmt, the
*				  printf-specification string.
*
* Description	This function writes characters from a PRINTF-
*		specification to the current window.
*
*		This routine will scroll the window if necessary, and
*		will also wrap text from one row to the next.
*
*		An error occurs if no window is designated as current.
*
* Returns	b_pcurwin->cur_loc     Possibly altered.
*		b_wnerr 	       Possible values:
*				(No change)	 Success.
*				WN_BAD_WIN	 b_pcurwin is invalid.
*				WN_NULL_PTR	 Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include <bwindow.h>


		/* Buffer to VSPRINTF into.  It is NIL the first    */
		/* time WNPRINTF is called, unless the user called  */
		/* WNSETBUF.					    */
char *b_wnprbf = NIL;

		/* Size of buffer.  Zero to start with. 	    */
unsigned int b_wnbfsz = 0;


int wnprintf (char *fmt,...)
{
    int 	 result;
    unsigned int size;
    va_list	 parg;

		/* Exit if current window is not valid. 	    */
    if (NIL == wnvalwin (b_pcurwin))
    {
	wnerror (WN_BAD_WIN);
	return (EOF);
    }

		/* Allocate the buffer for VSPRINTF to stuff	    */
		/* characters into.				    */
    if (b_wnprbf == NIL)
    {
	size = wnsetbuf (B_WNPRBF_SIZE);
	if (size != B_WNPRBF_SIZE)
	    wnerror (WN_NO_MEMORY);
    }

		/* Make parg point to the parameter just before the */
		/* beginning of the variable argument list.	    */
    va_start (parg, fmt);

		/* Put a NUL at the end of the VSPRINTF buffer so   */
		/* we know if it overflowed.			    */
    b_wnprbf [b_wnbfsz - 1] = '\0';

		/* Do the actual interpretation of the format	    */
		/* string into B_WNPRBF.			    */
    result = vsprintf (b_wnprbf, fmt, parg);

		/* Done using the variable argument list-discard it.*/
    va_end (parg);

		/* Check for overflow.				    */
    if (b_wnprbf [b_wnbfsz - 1] != '\0')
    {
	wnerror (WN_PR_OVERFLOW);
	return (EOF);
    }

		/* If VSPRINTF didn't get an error, send the string */
		/* to the window.				    */
    if (result > 0)
	wnwrstrn (b_pcurwin, b_wnprbf, result, -1, -1, WN_UPDATE);

    return (result);
}
