/**
*
* Name		WNREMOVE -- Remove window from screen.
*
* Synopsis	presult = wnremove (pwindow);
*
*		BWINDOW *presult  Pointer to newly-removed BWINDOW
*				  structure, or NIL if failure.
*		BWINDOW *pwindow  Pointer to BWINDOW structure to
*				  remove.
*
* Description	This function removes a window that is currently
*		displayed and restores the previous contents of the
*		screen.
*
*		An error occurs if pwin does not point to a valid window
*		structure, or if the window has been designated "not
*		removable", or if the window is not currently displayed.
*
*		Use WNFORGET to detach a non-removable window from the
*		video display page where it is displayed.
*
* Returns	presult 	  Pointer to newly-removed BWINDOW
*				  structure, or NIL if failure.
*		*pwindow	  Several fields altered.
*		b_pactnode[][]	  Window node with active cursor.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   *pwin is invalid.
*				  WN_NOT_SHOWN	   Not currently shown.
*				  WN_CANT_HIDE	   Not removable.
*				  WN_BAD_NODE	   Internal error.
*				  WN_BAD_DEV	   Internal error.
*				  WN_BAD_PAGE	   Internal error.
*				  WN_ILL_DIM	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


BWINDOW *wnremove (pwindow)
BWINDOW *pwindow;
{
    wnvalidw (pwindow)

		/* Ensure it's showing.                             */
    if (pwindow->where_shown.dev != COLOR &&
	pwindow->where_shown.dev != MONO)
	wnreterr (WN_NOT_SHOWN);

		/* Remove from physical screen. 		    */
    if (NIL == wnhide (pwindow))
	return (NIL);

		/* Remove from linked list.			    */
    if (NIL == wnpgrem (pwindow))
	return (NIL);

		/* Mark as not shown.				    */
    pwindow->where_shown.dev =
    pwindow->where_prev.dev  = ABSENT;
    pwindow->options.hidden  = 0;

		/* If this window is current, mark no window current*/
    if (b_pcurwin == pwindow)
	b_pcurwin = NIL;

    return (pwindow);
}
