/**
*
* Name		WNSCROLL -- Vertically scroll the current window.
*
* Synopsis	presult = wnscroll (num_rows, fore, back, dir);
*
*		BWINDOW *presult  Pointer to newly-scrolled BWINDOW
*				  structure, or NIL if failure.
*		int num_rows	  Number of lines to scroll.  A value of 0
*				  specifies that all lines within the
*				  window should be scrolled (thus clearing
*				  the window).
*		int fore	  Foreground attribute of new blank lines.
*				  (-1 specifies window's native foreground
*				  color).
*		int back	  Background attribute of new blank lines.
*				  (-1 specifies window's native background
*				  color).
*		int dir 	  Scrolling direction (SCR_UP or SCR_DOWN).
*
* Description	This function moves rows of characters (with their
*		attributes) up or down within the current window.  The
*		vacant rows are filled with blanks and a specified
*		attribute.
*
*		An error occurs if no window is current.
*
* Returns	presult 	  Pointer to newly-scrolled BWINDOW
*				  structure, or NIL if failure.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   b_pcurwin is invalid.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>

#define H (b_pcurwin->img.dim.h)
#define W (b_pcurwin->img.dim.w)


BWINDOW *wnscroll (num_rows, fore, back, dir)
int	 num_rows, fore, back, dir;
{
		/* Validate window data structures.		    */
    wnvalidw (b_pcurwin)

		/* Force num_rows to be 0 (clear window) if not in  */
		/* range.					    */
    if (utrange (num_rows, 1, H))
	num_rows = 0;

    if (dir == SCR_UP)
	return (wnscrblk (b_pcurwin,
			  0,	  0,
			  H - 1,  W - 1,
			  fore, back, WNSCR_UP, num_rows, WN_UPDATE));

    return (wnscrblk (b_pcurwin,
		      0,      0,
		      H - 1,  W - 1,
		      fore, back, WNSCR_DOWN, num_rows, WN_UPDATE));
}
