/**
*
* Name		WNSELDEV -- Validate & select video page on which
*			    to display a rectangular region of data.
*
* Synopsis	ercode = wnseldev (pwhere, pdim, poldpage);
*
*		int   ercode	  0 if successful, nonzero if not.
*		WHERE *pwhere	  Pointer to WHERE structure denoting
*				  device, display page, and coordinates
*				  where data may be displayed.
*		DIM   *pdim	  Pointer to DIM structure denoting
*				  dimensions of rectangle.
*		int   *poldpage   Pointer to returned value:  previous
*				  current page on selected device.
*				  (This is invalid if ercode is nonzero.)
*
* Description	This function ensures that a requested video device and
*		display page are present and large enough to contain a
*		rectangular region.  If they are, then the device and
*		page are made current; if not, then an error is reported
*		via a nonzero return value.
*
* Returns	ercode		  0 if successful, nonzero if not.
*		b_device	  Newly-selected device (if successful).
*		b_curpage	  Newly-selected page (if successful).
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


int wnseldev (pwhere, pdim, poldpage)
WHERE *pwhere;
DIM   *pdim;
int   *poldpage;
{
    int old_dev, mode, columns, act_page;

		/* Save former device.				    */
    old_dev = scmode (&mode, &columns, &act_page);

		/* Absent device.				    */
    if (scchgdev (pwhere->dev) != 0)
	return (1);

		/* Former current page on new device.		    */
    *poldpage = b_curpage;

		/* If left edge, top row, right edge, bottom row,   */
		/* or page is incorrect, detect it.		    */
    if (pwhere->corner.row < 0			||
	pwhere->corner.col < 0			||
	pwhere->corner.col + pdim->w > columns	||
	pwhere->corner.row + pdim->h > scrows() ||
	pwhere->page >= scpages())
    {
		/* Error:  Restore former device and page.	    */
	scchgdev (old_dev);
	return (1);
    }

    scpage (pwhere->page);

    return (0);
}
