/**
*
* Name		WNSETBUF -- Allocate buffer space for WNPRINTF.
*
* Synopsis	retsize = wnsetbuf (size);
*
*		unsigned retsize  The current size of the WNPRINTF
*				  buffer.
*		unsigned size	  The requested new size for the
*				  WNPRINTF buffer.
*
* Description	This function allocates a buffer for WNPRINTF to use
*		as scratch space.
*
* Returns	retsize 	New buffer size.
*		b_wnbfsz	Also new buffer size.
*		b_wnprbf	Pointer to buffer.
*		b_wnerr 	Possible values:
*				(No change)	 Success.
*				WN_NO_MEMORY	 Insufficient memory.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/

#include <stdio.h>

#include <bwindow.h>

unsigned wnsetbuf (size)
unsigned size;
{
		/* If requested size is same as existing size, OK.  */
    if (size == b_wnbfsz)
	return (size);

		/* Free the buffer if it exists.		    */
    if (b_wnprbf != NIL)
	free (b_wnprbf);

		/* Allocate a new buffer -- register a window error */
		/* if there is a problem.			    */
    if ((size != 0) &&
	((b_wnprbf = calloc (size, sizeof (char))) == NULL))
	wnreterz (WN_NO_MEMORY);

		/* Return requested size.			    */
    return (b_wnbfsz = size);
}
