/**
*
* Name		WNUPDATE -- Write pending I/O requests to a window.
*
* Synopsis	presult = wnupdate (pwin);
*
*		BWINDOW *presult  Pointer to BWINDOW structure updated,
*				  or NIL if failure.
*		BWINDOW *pwin	  Pointer to BWINDOW structure to
*				  update.
*
* Description	This function writes any pending I/O requests to the
*		designated window if there are any waiting.  (Nothing
*		will occur if any portion of the window's data area is
*		covered by another window.)
*
*		Also updates cursor.
*
*		This function does not designate the window "not
*		delayed".  Use WNSETOPT to designate a window "delayed"
*		or "not delayed".
*
*		An error occurs if pwin does not point to a valid window
*		structure or if the window is not currently displayed.
*
* Returns	presult 	  Pointer to newly-updated BWINDOW
*				  structure, or NIL if failure.
*		*pwin		  Several fields altered.
*		b_wnerr 	  Possible values:
*				  (No change)	   Success.
*				  WN_BAD_WIN	   *pwin is invalid.
*				  WN_NOT_SHOWN	   Not currently shown.
*				  WN_BAD_DEV	   Internal error.
*				  WN_ILL_DIM	   Internal error.
*				  WN_NULL_PTR	   Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1986, 1987
*
**/


#include <bwindow.h>


BWINDOW *wnupdate (pwin)
BWINDOW *pwin;
{
    int old_dev, old_page, old_npage;
    int mode, columns, act_page;
    BWINDOW *presult;

		/* Validate window data structures.		    */
    wnvalidw (pwin)

		/* Quit if invisible, if already up-to-date, or if  */
		/* frozen.					    */
    if (pwin->options.hidden	 ||
	(!pwin->internals.dirty) ||
	pwin->internals.frozen)
	return (pwin);

		/* Note former device, current page on former device*/
    old_dev  = scmode (&mode, &columns, &act_page);
    old_page = b_curpage;

		/* Validate and select device and page. 	    */
    if (wnseldev (&pwin->where_shown, &pwin->img.dim, &old_npage))
	wnreterr (WN_NOT_SHOWN);

    presult = pwin;
    if (wnputimg (&pwin->img, &pwin->where_shown) == NIL)
	presult = NIL;

    else
    {
	pwin->internals.dirty = 0;

				/* Move physical cursor.	    */
	if (!pwin->internals.cur_frozen)
	    sccurset (pwin->where_shown.corner.row + b_pcurwin->cur_loc.row,
		      pwin->where_shown.corner.col + b_pcurwin->cur_loc.col);
    }

		/* Restore current page on new device; restore old  */
		/* device; restore current page on old device.	    */
    scpage (old_npage);
    scchgdev (old_dev);
    scpage (old_page);

    return (presult);
}
