/**
*
* Name		WNWRTTYX -- Write a character to a window TTY-style.
*
* Synopsis	wnwrttyx (pwin, ch, fore, back, option);
*
*		BWINDOW *pwin	  Window to write character into.
*		char	 ch	  Character to write.
*		int	 fore	  -1 if default foreground attribute is to
*				  be used; a value between 0 and 15
*				  specifies a new foreground attribute.
*		int	 back	  -1 if default background attribute is to
*				  be used; a value between 0 and 15
*				  specifies a new background attribute.
*		int	 option   The bitwise OR-ing of the following
*				  values:
*					WN_UPDATE
*					    Update window when through.
*					WN_NO_UPDATE
*					    Do not update window when
*					    through.
*					WN_SCROLL
*					    Scroll the window when the end
*					    of the last line is reached.
*					WN_NO_SCROLL
*					    Don't scroll the window.
*					    characters written after the
*					    end of the window buffer are
*					    lost.
*					WN_LF_LF
*					    Treat CR as CR, LF as LF--
*					WN_LF_CRLF
*					    Treat CR as CR, LF as CR/LF--
*					CHARS_ONLY
*					    Expect buffer to contain
*					    only characters.
*					CHAR_ATTR
*					    Expect (char,attr) pairs
*					    rather than just characters.
*
* Description	This function writes a character to the window in the
*		manner of a teletype.  It treats carriage return, line
*		feed, backspace, and bell as commands rather than as
*		printable characters.  All output and scrolling are
*		confined to the window.
*
*		The window's cursor is left after the displayed
*		character.  The visible cursor is not moved unless this
*		window is currently selected (via WNCURSOR) to have this
*		page's active cursor.
*
*		The window is scrolled if a line feed occurs on the last
*		line of the window or if the last line is overflowed.
*		The window's native attributes are used for the new
*		blank row.
*
*		If the screen is in a text mode, specifying -1 for fore
*		or back uses the window's default foreground or
*		background attributes, respectively.  If the screen is
*		in a graphics mode, then specifying -1 for fore will
*		cause color 1 to be used.
*
* Special	Tab characters ('\t') are displayed literally.
* characters
*		NUL ('\0') is a printable character (printed as a blank
*		space).
*
*		Line feed ('\n') causes the cursor to move down one
*		row, unless it is already on the bottom row of the
*		window, in which case the window is scrolled.
*
*		Carriage return ('\r') causes the cursor to move to the
*		leftmost column of the window.
*
*		Backspace ('\b') causes the cursor to move one column to
*		the left (non-destructively), unless it is already at
*		the leftmost column of the window, in which case nothing
*		happens.
*
*		The BEL character ('\a') causes the computer's bell to
*		sound if the current page is active.
*
* Returns	(Function return type is void.)
*		b_wnerr     Possible values:
*			    (No change)     Success.
*			    WN_BAD_WIN	    *pwin invalid.
*			    WN_ILL_DIM	    Internal error.
*
* Version	5.00 (C)Copyright Blaise Computing Inc. 1987
*
**/


#include <bwindow.h>


void	 wnwrttyx (pwin, ch, fore, back, option)
BWINDOW *pwin;
char	 ch;
int	 fore, back, option;
{
    char s [1];

    s [0] = ch;
    wnwrstrn (pwin, s, 1, fore, back, option);
}
