
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <assert.h>
#include <termios.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include "srec.h"

#define NODEBUG

struct hdrIN {
    unsigned char x1, x2, x3, x4;
    };

struct hdrOUT {
    unsigned char x1, x2, x3, x4;
    unsigned char data[ 4096 ];
    };

int
ads_receive( int ads, void* buf, int size )
{
    static struct hdrIN hdrIN;
    int rc;

    rc = read( ads, &hdrIN, 4 );
    if ( rc < 0 ) {
        perror( "ads read" );
        sleep( 1 );
        return rc;
        }

    if ( rc == 0 )
        return rc;

#ifdef DEBUG
{
    int i;
    fprintf( stderr, "<got: %d bytes: ", rc );
    fprintf( stderr, "hdrIN: %d %d %d %d>\n", hdrIN.x1, hdrIN.x2, hdrIN.x3, hdrIN.x4 );
    }
#endif /* DEBUG */

    if ( rc != 4 || hdrIN.x1 != 2 || hdrIN.x2 != 0 )
        return 0;

    rc = hdrIN.x4;
    assert( rc <= size ); 

    rc = read( ads, buf, rc );

#ifdef DEBUG
{
    int i;
    fprintf( stderr, "<got: %d bytes of %d: ", rc, hdrIN.x4 );
    for ( i = 0; i < rc; i++ )
        fputc( ((char*)buf)[ i ], stderr );
    fprintf( stderr, ">\n" );
    }
#endif /* DEBUG */

    return rc;
    }

int
ads_send( int ads, void* buf, int size )
{
    static struct hdrOUT hdrOUT;
    int rc;

    hdrOUT.x1 = 2;
    hdrOUT.x2 = 0;
    hdrOUT.x3 = ( size >> 8 ) & 0xff;
    hdrOUT.x4 = size & 0xff;

    memcpy( hdrOUT.data, buf, size );

    rc = write( ads, &hdrOUT, 4 + size );

    if ( rc < 0 ) {
        perror( "ads write" );
        sleep( 1 );
        }

    return rc;
    }

struct termios old_tios;

void setup_term ()
{
    struct termios tios;
    tcgetattr( 0, &old_tios );

    tcgetattr( 0, &tios );
    tios.c_lflag &= ~ICANON;
    tios.c_lflag &= ~ECHO;
    tios.c_iflag &= ~ICRNL;
    tios.c_oflag &= ~OCRNL;
    tcsetattr( 0, 0, &tios );
    }

void restore_term ()
{
    tcsetattr( 0, 0, &old_tios );
    }

char srec_filename[ 80 ];

char o_line[ 4096 ];
int wait_PDL = -1;
int o_linep = 0;

char buf[ 4096 ];

void try_PDL ();
int ads;

int main () 
{
    int i;
    int rc;

    ads = open( "/dev/adi0", O_RDWR );

    setup_term ();

    ads_send( ads, "ver\n", 4 );

    o_linep = 0;

    for ( ;; ) {

        rc = ads_receive( ads, buf, sizeof( buf ) ); 

        if ( rc > 0 ) {

            for ( i = 0; i < rc; i++ ) {
                if ( o_linep >= sizeof( o_line ) ) {
                    if ( o_line[ o_linep - 1 ] == '\n' )
                        o_line[ o_linep - 9 ] = '\n';
                    o_linep = o_linep - 10;
                    }
                o_line[ o_linep++ ] = buf[ i ];
                putchar( buf[ i ] );
                }
            fflush( stdout );

            o_line[ o_linep ] = 0;

            if ( wait_PDL == 1
            && o_linep >= 5 
            && strcmp( o_line + o_linep - 5, "PDL\n\r" ) == 0 ) {
                printf( "\033[A" ); fflush( stdout );
                psend_srec_file( ads, srec_filename, 0, 0L );
                wait_PDL = 2;
                }

            if ( o_line[ o_linep - 2 ] == '\n' && o_line[ o_linep - 1 ] == '\r' ) {
                o_linep = 0;
                o_line[ 0 ] = 0;
                }
            }
        else if ( rc == 0 ) {

            struct timeval timeout;
            long readfds;

            if ( o_linep >= 13
            && strcmp( o_line + o_linep - 13, "QUICCbug>Q\b \b" ) == 0 ) {
                if ( wait_PDL == 2  ) {
                    ads_send( ads, "rm pc\n404000.\n", 14 );
                    wait_PDL = -1;
                    continue;
                    }
                else if ( wait_PDL == -1 ) {
                    wait_PDL = 0; 
                    continue;
                    }
                }

            readfds = 1; /* mask fd 0 */
            timeout.tv_sec = 1;
            timeout.tv_usec = 0;
            if ( 1 == select( 1, (fd_set*)&readfds, NULL, NULL, &timeout ) ) {
                char ch; 
                read( 0, &ch, 1 ); 
#ifdef DEBUG
                fprintf( stderr, "<keyboard: %d>\n", ch );
#endif /* DEBUG */

                switch ( ch ) {

                    case 'A'-'@': /* Ctrl-A: Abort */
                        ioctl( ads, 1, 0 );
                        break;

                    case 'R'-'@': /* Ctrl-R: Soft Reset */
                        ioctl( ads, 2, 0 );
                        break;

                    case 'Y'-'@': /* Ctrl-Y: Hard Reset */
                        ioctl( ads, 3, 0 );
                        break;

                    case 'I'-'@': /* TAB: PDL */
                        try_PDL ();
                        break;

                    case 'X'-'@': /* Ctrl-X or Escape: Quit */
                    case 27:
                        goto EXIT;

                    default: /* Send this character to ADS board */
                    {
                        char sendbuf[ 1 ] = { ch };
                        ads_send( ads, sendbuf, 1 );
                        }
                        wait_PDL = -1; /* disable PDL tab option also */
                        break;
                    }
                }

            else { /* Poll ads */
                ads_send( ads, "", 0 );
                }
            }
        }

    EXIT:
    close( ads );
    restore_term ();

    printf( "\n" );
    return 0;
    }

void try_PDL ()
{
    FILE* f = NULL;

    if ( wait_PDL != 0 )
        return;

    printf( "\nEnter SREC filename: " );
    restore_term ();
    gets( srec_filename );
    setup_term ();

    if ( ! srec_filename[ 0 ] ) {
        printf( o_line );
        fflush( stdout );
        return;
        }

    f = fopen( srec_filename, "rb" );

    if ( ! f ) {
        printf( "Error opening file: %s\n", srec_filename );
        printf( o_line );
        fflush( stdout );
        return;
        } 

    fclose( f );

    ads_send( ads, "PDL\n", 4 );
    wait_PDL = 1;

    return;
    }

