
#define NODEBUG

#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "srec.h"

/* ------------------- definitions used in S-records */

enum {
    S0, S1, S2, S3, S4, S5, S6, S7, S8, S9
    };

typedef struct {
    int   type;
    long  address;
    int   length;
    } srec;

#define MAXIMUM 0x2000

/* ------------------ structure used by host and board for psend */

/* bit masks for flags field */
#define ENTRY_POINT     0x00000008      /* entry point in address  */
#define LAST_PKT        0x00000002
#define GO              0x00000004      /* don't return to bug at end  */

typedef struct {
    unsigned long flags;
    unsigned long address;
    unsigned long length;               /* length of data */
    } PHDR;

typedef struct {
    PHDR            hdr;
    unsigned char   data[ MAXIMUM ];
    } PSEND;

static int             ads_board = -1;

static unsigned long   entry_point = -1;

static int             RelocAdd = 0;
static unsigned long   BaseReloc = 0;

static PSEND           sect;

/* ------------------ declaration of helper procedures */

static void swapphdr( register PHDR* phdr );
static int BadTarget( char* s);
static int allHex( char* s );
static int hex( unsigned char c );
static int readadr( unsigned char line[], unsigned long* l, int n );
static int SRECparse( FILE* f, srec* s, unsigned char* data );

void
psend_srec_file( int adi, char* filename, int reloc, unsigned long base )
{
    FILE *f;                /* input file descriptor */
    static srec srec;
    int i, n;
    long fsize = 0, curp = 0;
    long bytecount = 0;

    ads_board = adi;
    RelocAdd = reloc;
    BaseReloc = base;

    /* f is used to read file header */
    f = fopen( filename, "rb" );
    if ( ! f ) {
        perror( filename );
        return;
        }

    fseek( f, 0L, SEEK_END );
    fsize = ftell( f );
    fseek( f, 0L, SEEK_SET );

    printf( "Transferring file %s.\n", filename );

    sect.hdr.flags = 0;

    for (;;) {
        i = SRECparse( f, &srec, sect.data );

        if( i == 0 )
            continue;
        else if ( i < 0 )
            break;

        curp += i;

        printf( " %02ld%%\r", curp * 100 / fsize ); fflush( stdout );

        if ( RelocAdd )
            srec.address += BaseReloc;
        else
            srec.address -= BaseReloc;

#ifdef DEBUG
        printf( "parsed: adr = %lx, length = %x\n",
                srec.address, srec.length);
#endif /* DEBUG */

        sect.hdr.address = srec.address;
        n = sect.hdr.length = srec.length;

        swapphdr( &sect.hdr );
        i = write( ads_board, &sect, n + sizeof( PHDR ) );
        swapphdr( &sect.hdr );

        if ( i < n ) {
            perror( "write" );
            fclose( f );
            return;
            }

        bytecount += i;
        }

    sect.hdr.flags = LAST_PKT;
    sect.hdr.address = 0; 
    sect.hdr.length = 0;

    if ( entry_point >= 0 )  {
        printf( "Entry point is 0x%lx.\n", entry_point );

        sect.hdr.flags |= ENTRY_POINT;

        if( RelocAdd )
            sect.hdr.address = BaseReloc + entry_point;
        else
            sect.hdr.address = BaseReloc - entry_point;

/*
        if ( go )
            sect.hdr.flags |= GO;
*/
        }

    swapphdr( &sect.hdr );
    i = write( ads_board, &sect, sizeof( PHDR ) );
    swapphdr( &sect.hdr );

    if ( i < 1 ){
        perror( "write(end)" );
        fclose( f );
        return;
        }

    bytecount += i;

    fclose( f );

    printf( "Transferred %ld bytes.\n", bytecount );
    }

/* ------------------ definitons of helper procedures */

static int
BadTarget( char* s)
{
    fprintf( stderr, "Bad target address: %s\n", s );
    exit( 1 );
    }

/*
 * checks if an address is given.
 */
static int
allHex( char* s )
{
    while( *s ){
        if( ! isxdigit( *s ) )
            return 0;
        s++;
        }
    return 1;
    }

static int
hex( unsigned char c )
{
    if( isdigit(c) )
        return c - '0';
    if( isupper(c) )
        return 10 + c - 'A';
    if( islower(c) )
        return 10 + c - 'a';
    return -1;     
    }

inline
static void
swapshort( unsigned short* adr )
{
    unsigned short a, b;

    a = (*adr & 0xff00) >> 8;
    b = (*adr & 0x00ff);
    *adr = (b << 8) | a; 
    }

inline
static void
swaplong( unsigned long* adr )
{
    unsigned long a, b, c, d;

    a = (*adr & 0xff000000) >> 24;
    b = (*adr & 0x00ff0000) >> 16;
    c = (*adr & 0x0000ff00) >> 8;
    d = (*adr & 0x000000ff);
    *adr = (d << 24) | (c << 16) | (b << 8) | a;
    }

static void
swapphdr( register PHDR* phdr )
{
    swaplong( &phdr->flags );
    swaplong( &phdr->address );
    swaplong( &phdr->length );
    }

static int
readadr( unsigned char line[], unsigned long* l, int n )
{
    register int i;
    unsigned char data;
    long res;
    int nyb1, nyb2;

    res = 0;
    i = 4;
    while( n-- ) {
        nyb1 = hex( line[ i ] );
        nyb2 = hex( line[ i + 1 ] );
        data = ( nyb1 << 4 ) | nyb2;
        res = res << 8;
        res |= data;
        i += 2;
        }
    *l = res;
    return 1;
    }

static int
SRECparse( FILE* f, srec* s, unsigned char* data)
{
    register int i, j;
    static unsigned char line[ 512 ];
    int alen = 0;   /* address length */
    int nyb1, nyb2;
    static char warning[] = "Warning: %s is not an S-record (discarded)\n";

    if ( ! fgets( line, sizeof( line ), f ) ) {
        if( ferror( f ) )
            perror( "read" );
        return -1;
        }

    if( ( line[ 0 ] != 'S' ) && ( line[ 0 ] != 's' ) ) {
        fprintf( stderr, warning, line );
        return 0;
        }

    s->type = hex( line[ 1 ] );
    s->address = 0;
    s->length = 0;

    switch( line[ 1 ] ) {
    case '0':
    case '5':
        return 0;
    case '1':
        if( ! readadr( line, &s->address, 2 ) )
            return 1;
        alen = 2;
        break;
    case '2':
        if( ! readadr( line, &s->address, 3 ) )
            return 1;
        alen = 3;
        break;
    case '3':
        if( ! readadr( line, &s->address, 4 ) )
            return 1;
        alen = 4;
        break;
    case '4':
    case '6':
        fprintf( stderr, warning, line );
        return 0;
    case '7':
        if ( ! readadr( line, &entry_point, 4 ) )
            return 1;
        return 0;
    case '8':
    case '9':
        readadr( line, &entry_point, 0 );
        return 0;
    default:
        fprintf( stderr, warning, line );
        return 0;
        }

    /* we should only reach here for S1, S2 and S3 records */

    nyb1 = hex( line[ 2 ] );
    nyb2 = hex( line[ 3 ] );
    s->length = ( nyb1 << 4 ) | nyb2;
    s->length -= alen;      /* discard address bytes  */
    s->length--;            /* ignore checksum at end  */
    i = 4 + ( 2 * alen );   /* index of first data nybble  */
    j = s->length;

    while( j-- ) {
        nyb1 = hex( line[ i ] );
        nyb2 = hex( line[ i + 1 ] );
        *data++ = ( nyb1 << 4 ) | nyb2;
        i += 2;
        }

    return strlen( line );
    }
