/*				IO515.H

   Special header for the Siemems SAB 80515/80535 chips.
   Note that this file is supposed to be used with the
   in-line functions that are enabled with compiler command
   line option: -e

   For more information about the in-line functions, study
   file BIO51.C
*/


/* Pre-defined SFR byte addresses: */


#define   DAPR	   0xDA
#define   ADDAT	   0xD9
#define   ADCON    0xD8
#define   P0	   0x80
#define   P1	   0x90
#define   P2	   0xA0
#define   P3	   0xB0
#define   P4	   0xE8
#define   P5	   0xF8
#define   PSW	   0xD0
#define   ACC	   0xE0
#define   B	   0xF0
#define   SP	   0x81
#define   DPL	   0x82
#define   DPH	   0x83
#define   PCON	   0x87
#define   TCON	   0x88
#define   TMOD	   0x89
#define   TL0	   0x8A
#define   TL1	   0x8B
#define   TL2	   0xCC
#define   TH0	   0x8C
#define   TH1	   0x8D
#define   TH2	   0xCD
#define   IEN0	   0xA8
#define   IP0	   0xA9
#define   IEN1	   0xB8
#define   IP1	   0xB9
#define   IRCON	   0xC0
#define   CCEN	   0xC1
#define   CCL1	   0xC2
#define   CCH1	   0xC3
#define   CCL2	   0xC4
#define   CCH2	   0xC5
#define   CCL3	   0xC6
#define   CCH3	   0xC7
#define   CRCL	   0xCA
#define   CRCH	   0xCB
#define   T2CON    0xC8
#define   SCON	   0x98
#define   SBUF	   0x99

  
/* Pre-defined SFR bit addresses.
   The reason for the "bit" name-extension is just to
   clearly separate bit addresses from other names. */

/*========ADCON=======*/

#define   BD_bit	  0xDF
#define   CLK_bit	  0xDE

#define   BSY_bit	  0xDC
#define   ADM_bit	  0xDB
#define   MX2_bit	  0xDA
#define   MX1_bit	  0xD9
#define   MX0_bit	  0xD8

/*========IRCON=======*/

#define   EXF2_bit	  0xC7
#define   TF2_bit	  0xC6
#define   IEX6_bit	  0xC5
#define   IEX5_bit	  0xC4
#define   IEX4_bit	  0xC3
#define   IEX3_bit	  0xC2
#define   IEX2_bit	  0xC1
#define   IADC_bit  	  0xC0

/*========T2CON=======*/

#define   T2PS_bit	  0xCF
#define   I3FR_bit	  0xCE
#define   I2FR_bit	  0xCD
#define   T2R1_bit	  0xCC
#define   T2R0_bit	  0xCB
#define   T2CM_bit	  0xCA
#define   T2I1_bit	  0xC9
#define   T2I0_bit	  0xC8

/*========PSW=========*/

#define   CY_bit	  0xD7
#define   AC_bit	  0xD6
#define   F0_bit	  0xD5
#define   RS1_bit	  0xD4
#define   RS0_bit	  0xD3
#define   OV_bit	  0xD2
#define   F1_bit	  0xD1
#define   P_bit	  	  0xD0

/*========TCON========*/

#define   TF1_bit	  0x8F
#define   TR1_bit	  0x8E
#define   TF0_bit	  0x8D
#define   TR0_bit	  0x8C
#define   IE1_bit	  0x8B
#define   IT1_bit	  0x8A
#define   IE0_bit	  0x89
#define   IT0_bit	  0x88

/*========IEN0========*/

#define   EAL_bit	  0xAF
#define   WDT_bit	  0xAE
#define   ET2_bit	  0xAD
#define   ES_bit	  0xAC
#define   ET1_bit	  0xAB
#define   EX1_bit	  0xAA
#define   ET0_bit	  0xA9
#define   EX0_bit	  0xA8

/*========IEN1========*/

#define   EXEN2_bit	  0xBF
#define   SWDT_bit	  0xBE
#define   EX6_bit	  0xBD
#define   EX5_bit	  0xBC
#define   EX4_bit	  0xBB
#define   EX3_bit	  0xBA
#define   EX2_bit	  0xB9
#define   EADC_bit	  0xB8

/*========P0=========*/

#define   P0_7_bit	  0x87
#define   P0_6_bit	  0x86
#define   P0_5_bit	  0x85
#define   P0_4_bit	  0x84
#define   P0_3_bit	  0x83
#define   P0_2_bit	  0x82
#define   P0_1_bit	  0x81
#define   P0_0_bit	  0x80

/*========P1=========*/

#define   P1_7_bit	  0x97
#define   P1_6_bit	  0x96
#define   P1_5_bit	  0x95
#define   P1_4_bit	  0x94
#define   P1_3_bit	  0x93
#define   P1_2_bit	  0x92
#define   P1_1_bit	  0x91
#define   P1_0_bit	  0x90

/*========P2=========*/

#define   P2_7_bit	  0xA7
#define   P2_6_bit	  0xA6
#define   P2_5_bit	  0xA5
#define   P2_4_bit	  0xA4
#define   P2_3_bit	  0xA3
#define   P2_2_bit	  0xA2
#define   P2_1_bit	  0xA1
#define   P2_0_bit	  0xA0

/*========P3=========*/

#define   RD_bit	  0xB7
#define   WR_bit	  0xB6
#define   T1_bit	  0xB5
#define   T0_bit	  0xB4
#define   INT1_bit	  0xB3
#define   INT0_bit	  0xB2
#define   TXD_bit	  0xB1
#define   RXD_bit	  0xB0

#define   P3_7_bit	  0xB7
#define   P3_6_bit	  0xB6
#define   P3_5_bit	  0xB5
#define   P3_4_bit	  0xB4
#define   P3_3_bit	  0xB3
#define   P3_2_bit	  0xB2
#define   P3_1_bit	  0xB1
#define   P3_0_bit	  0xB0

/*========P4=========*/

#define   P4_7_bit	  0xEF
#define   P4_6_bit	  0xEE
#define   P4_5_bit	  0xED
#define   P4_4_bit	  0xEC
#define   P4_3_bit	  0xEB
#define   P4_2_bit	  0xEA
#define   P4_1_bit	  0xE9
#define   P4_0_bit	  0xE8

/*========P5=========*/

#define   P5_7_bit	  0xFF
#define   P5_6_bit	  0xFE
#define   P5_5_bit	  0xFD
#define   P5_4_bit	  0xFC
#define   P5_3_bit	  0xFB
#define   P5_2_bit	  0xFA
#define   P5_1_bit	  0xF9
#define   P5_0_bit	  0xF8

/*========SCON========*/

#define   SM0_bit	  0x9F
#define   SM1_bit	  0x9E
#define   SM2_bit	  0x9D
#define   REN_bit	  0x9C
#define   TB8_bit	  0x9B
#define   RB8_bit	  0x9A
#define   TI_bit	  0x99
#define   RI_bit	  0x98

