/*
 * ManufSectServlet.java
 *
 * Retrieves the list of all manufacturers and sections requiered 
 * for the productCatalog applet and performs the specified search in the Catalog DB
 *
 * Created on August 3, 2002, 3:04 PM
 *
 */

package micronet;

import javax.servlet.*;
import javax.servlet.http.*;
import java.sql.*;
import java.io.*;
import java.util.*;

/**
 *
 * @author  milos
 * @version
 */
public class CatalogServlet extends HttpServlet {
    
    
    private static final String jdbcDriver= "org.sourceforge.jxdbcon.JXDBConDriver";
    private static final String dbUrl= "jdbc:postgresql:net//localhost:5432/milos";    
    
    /** Initializes the servlet.
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
          
    }
    
    /** Destroys the servlet.
     */
    public void destroy() {
        
    }
    
    
    /** Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }
    
    /** Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }
    
    /** Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
      
        ReplyMessage result=new ReplyMessage();
        
        // open DB connection
        Connection con=null;
        Statement s=null;
        ResultSet rs=null;
        
        try {
         Class.forName(jdbcDriver);
         con = DriverManager.getConnection(dbUrl,"www","www");
        }catch(Exception e){
            e.printStackTrace();
            con=null;
        }
        
        // depending on type of request we decide what to do
        String type=request.getParameter("type");        
        if(type.equalsIgnoreCase("1")){  // get list of manufacturers and sections
        
            try {
                // how many sections
                s=con.createStatement();
                rs=s.executeQuery("select count(*) from sections where sections.visible=true");
                rs.next();                       
                String secArray[]=new String[rs.getInt(1)];
                // how many manufacturers + get all manufacturers
                rs=s.executeQuery("select count(*) from manufacturers");
                rs.next();
                int dim=rs.getInt(1);
                String manArray[]=new String[dim];
                int[] mid=new int[dim];
                int[] secPerManArray=new int[dim];
                rs=s.executeQuery("select name,id from manufacturers");
                int i=0;
                while(rs.next()){                
                    manArray[i]=rs.getString(1);
                    mid[i++]=rs.getInt(2);
                }
                // store manufacturers in reply message
                result.manufacturerNames=manArray;
                // now get sections with respect to manufacturers and record numOfSec per manuf in reference array
                int sectionIndex=0;
                for(int j=0;j<manArray.length;j++){
                    int numPerMan=0;
                    rs=s.executeQuery("select name_en from sections where sections.visible=true and manuf_id="+mid[j]);
                    while(rs.next()){                
                        secArray[sectionIndex++]=rs.getString(1);
                        numPerMan++;
                    }
                    secPerManArray[j]=numPerMan;
                }
                // now store sections and their belongings to manufs in reply message
                result.sectionNames=secArray;
                result.secPerMan=secPerManArray;
            }catch(Exception e){result=null; e.printStackTrace();}
            
        
        
        }else { // perform searching
            
            
            String productPattern=request.getParameter("prodPat");
            String sectionName=request.getParameter("selName");
            String manufacturerName=request.getParameter("manName");
            String[] manufacturerNames;
            int[] orders;            
            String[] prodNames;
            String[] prices;
            String[] urls;
            
            // test query pattern and change it if necessary
            String q1,q2;
            if(sectionName.equals("All sections")) q2="";
            else q2=" sections.name_en like '"+sectionName+"' and";
            if(manufacturerName.equals("All manufacturers")) q1=""; 
            else q1=" manufacturers.name like '"+manufacturerName+"' and"; 
            
            // issue query
            try {
                s=con.createStatement();                
                rs=s.executeQuery("select distinct count(*) from manufacturers,sections,products where products.visible=true and "+q1+q2+  
                                  " (products.product_name like '%"+productPattern+"%' or products.order_no like '%"+productPattern+"%') and manufacturers.id=sections.manuf_id"+ 
                                  " and manufacturers.id=products.manuf_id and sections.section=products.section_no");                
                rs.next();
                int dim=rs.getInt(1);                
                manufacturerNames=new String[dim];
                orders=new int[dim];
                prodNames=new String[dim];
                prices=new String[dim];
                urls=new String[dim];                
                rs=s.executeQuery("select  distinct products.order_no,products.product_name,manufacturers.name,products.price,products.document,products.page_no"+ 
                                  " from manufacturers,sections,products where products.visible=true and "+q1+q2+  
                                  " (products.product_name like '%"+productPattern+"%' or products.order_no like '%"+productPattern+"%') and manufacturers.id=sections.manuf_id"+ 
                                  " and manufacturers.id=products.manuf_id and sections.section=products.section_no");                
                int i=0;
                while(rs.next()){                
                    orders[i]=rs.getInt(1);
                    prodNames[i]=rs.getString(2);                               
                    manufacturerNames[i]=rs.getString(3);
                    if((prices[i]=rs.getString(4))==null) prices[i]="Call";
                    if((urls[i]=rs.getString(5))==null) urls[i]="";
                    urls[i]+="#Page="+rs.getInt(6)+"&";
                    i++;
                }
                result.orders=orders;
                result.prodNames=prodNames;
                result.prices=prices;
                result.urls=urls;
                result.manufacturerNames=manufacturerNames;
                                
            }catch(Exception e){result=null; e.printStackTrace();}
            
        }
        
        // close DB connection
        try {
         if(con!=null) con.close();
        }catch(Exception e){
            con=null;
        }finally {
            con=null;
        }
        
        // write ReplyMessage object back to client side thru stream 
        if(result!=null){
        
           ObjectOutputStream out= new ObjectOutputStream(response.getOutputStream());
           out.writeObject(result);
           out.flush();
           out.close();           
        }
    }                    
}
