
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <conio.h>

#include "rtkernel.h"
#include "timer.h"
#include "rtcom.h"

#include "std.h"
#include "atime.h"
#include "nvb.h"
#include "comprs.h"
#include "acct.h"
#include "tarifer.h"

#include "sendexp.h"

#include "xcb.h"
#include "np.h"
#include "pktdrvr.h"
#include "nppkt.h"

nv_buffer nvb( 67000L );

const int MAIN_PRIO = 2;
const int SOCK_PRIO = MAIN_PRIO + 1;
const int NET_PRIO = MAIN_PRIO + 2; // +2, +3 and +4 are net priorities
const int TMERGE_PRIO = MAIN_PRIO + 5;
const int PBX_PRIO2 = MAIN_PRIO + 6;
const int PBX_PRIO1 = MAIN_PRIO + 7;
const int TCOLCT_PRIO = MAIN_PRIO + 8;

int PBX_SMDR_PORT = 0;
int PBX_PROG_PORT = 1;

Prog336 pp1, pp2;

extern unsigned _stklen = 30000u;
TPBX* tpbx = NULL;

npProtocol* shostess;
TaskHandle thostess;

// ============================================================= Hostess

static int nvb_get ()
{
    return nvb.get ();
    }

void hostess ()
{
    RTKFree8087 ();

    static char buf[ 80 ];
    npAddress from;
    long unack_idtime = 0;
    unsigned unack_idseq = 0;

    for ( ;; ) {
        shostess->Receive( from, buf, sizeof( buf ) );
        if ( strncmp( buf, "HOSTESS ", 8 ) == 0 ) {

            if ( strcmp( buf + 8, "GET ACCT" ) == 0 ) {
                if ( nvb.size () <= 0 ) {
                    uPrintf( "%s\r", (char*)aTime( time( NULL ) ) );
                    static char reply [] = "HOSTESS NODATA";
                    shostess->Send( from, reply, sizeof( reply ) );
                    }
                else {
                    uPrintf( "hostess: processing [%s] from [%s]\n",
                             buf + 8, (char*)ascii( from ) );

                    int type = 0;
                    while ( type >= 0 && type != 'A' )
                        type = nvb.get ();

                    if ( type != 'A' ) {
                        nvb.get_ack ();
                        uPrintf( "hostess: Panic: Bad magic\n" );
                        }
                    else {
                        acct_record acct;
                        decompress_struct( &acct, nvb_get );
                        acct.show ();
                        unack_idtime = acct.myTime;
                        unack_idseq = acct.seq_no;
                        shostess->Send( from, &acct, sizeof( acct_record ) );
                        }
                    }
                continue;
                }

            uPrintf( "hostess: processing [%s] from [%s]\n",
                     buf + 8, (char*)ascii( from ) );

            if ( strncmp( buf + 8, "ACK ", 4 ) == 0 ) {
                long idtime;
                unsigned idseq;
                sscanf( buf + 8 + 4, "%ld %u", &idtime, &idseq );
                if ( idtime == unack_idtime && idseq == unack_idseq ) {
                    nvb.get_ack ();
                    uPrintf( "hotsess: Data transfer acknowledged.\n" );
                    unack_idtime = 0;
                    unack_idseq = 0;
                    }
                }
            else if ( strncmp( buf + 8, "LOCK ", 5 ) == 0 ) {
                int port= 0;
                char dn[ 20 ] = { 0 };
                sscanf( buf + 8 + 5, "%d %s", &port, &dn );
                uPrintf( "hostess: Locking port %d, DN %s\n", port, dn );
                if ( ! pp1.test_ope_mode () ) {
                    pp1.enter_ope_mode( PBX_PROG_PORT, 150 );
                    if ( ! pp1.test_ope_mode () ) {
                        uPrintf( "hostess: Error entering OPE mode !\n" );
                        continue;
                        }
                    }
                pp1.disable_dn( dn );
                }
            else if ( strncmp( buf + 8, "UNLOCK ", 7 ) == 0 ) {
                int port = 0;
                char dn[ 20 ] = { 0 };
                sscanf( buf + 8 + 7, "%d %s", &port, &dn );
                uPrintf( "hostess: Unlocking port %d, DN %s\n", port, dn );
                if ( ! pp1.test_ope_mode () ) {
                    pp1.enter_ope_mode( PBX_PROG_PORT, 150 );
                    if ( ! pp1.test_ope_mode () ) {
                        uPrintf( "hostess: Error entering OPE mode !\n" );
                        continue;
                        }
                    }
                pp1.enable_dn( dn );
                }
            else {
                uPrintf( "hostess: Invalid request.\n" );
                }
            }
        }
    }

// ============================================================= Main

void main ()
{
    // Initialize Non-Volatile Transaction Buffer
    if ( ! nvb.has_arena () ) {
        uPrintf( "Cannot allocate space for NVB\n" );
        exit( EXIT_FAILURE );
        }

    if ( nvb.test_integrity () )
        uPrintf( "Allocated NVB arena has %ld bytes\n", nvb.freesize () );
    else {
        uPrintf( "panic: nvb integrity failure\n" );
        exit( EXIT_FAILURE );
        }

    lastcall_message::mempool.Configure( 64 );
    acctrec_message::mempool.Configure( 64 );

    new T16pN( 0x200, 8 );
    new T16pM( 0, "T1038" );
    new T16pM( 0, "T1039" );
    new T16pM( 0, "T1040" );
    new T16pM( 0, "T1041" );
    new T16pM( 0, "T1042" );
    new T16pM( 0, "T1043" );
    new T16pM( 0, "T1044" );
    new T16pM( 0, "T1045" );

    // Initialize TPBX (PBX parser) interface
    tpbx = new TPBX( "TPBX.CFG" );
    if ( ! tpbx->title )
        exit( EXIT_FAILURE );

    uPrintf( "Configured for: %s\n", tpbx->title );

    // Initialze RTKernel stuff

    RTKernelInit( MAIN_PRIO );
    TimerInit ();
    RTComInit ();

    if ( ! init_net( npAddress( "147.91.1.101:0" ), MAIN_PRIO + 1 ) )
        exit( EXIT_FAILURE );

    new npRoute( net, npAddress( "147.91.1.102:1" ),
                 npPkt->OutQ, npChannel( "10:AA:BB:CC:DD:01" ) );

    SetTimerInterval( 0.005 );

    InitPort( PBX_SMDR_PORT, 9600, PARITY_NONE, 1, 8 );
    EnableCOMInterrupt( PBX_SMDR_PORT, 1024 );

    InitPort( PBX_PROG_PORT, 9600, PARITY_NONE, 1, 8 );
    pp1.enter_ope_mode( PBX_PROG_PORT, 150 );
    if ( ! pp1.test_ope_mode () ) {
        uPrintf( "Warning: Prog336::pp1 not in OPE mode !\n" );
        }

    RTKCreateTask( PBXReceiver,    PBX_PRIO1,    2048, "PBX Receiver" );
    RTKCreateTask( T16pM::Merger,  TMERGE_PRIO,  2048, "T16pM Merger" );
    RTKCreateTask( T16pN::Collect, TCOLCT_PRIO,  2048, "T16pN Collector" );

    shostess = new npProtocol( net, npAddress( "*:1" ) );
    thostess = RTKCreateTask( hostess,  SOCK_PRIO,  2048, "S-hostess" );

    // End of Initialization
    uPrintf( "T16 initialization status O.K.\n" );

    for ( ;; ) {

        if ( kbhit () ) {
            int ch = getch ();

            if ( ch == 27 )
                break;
            else if ( ch == 'r' ) {
                uPrintf( "\n" );
                net->ShowRoutes ();
                }
            else if ( ch == 's' ) {
                uPrintf( "\n" );
                XCB_pool::Statistics ();
                }
            else if ( ch == 'i' ) {
                char buffer[ 256 ];
                RTKIRQInfo( buffer, 256 );
                uPrintf( buffer );
                }
            }
        }

    SetTimerInterval( 0.060 );

    outp( 0x3F8 + 4, 0  ); // turn off COM1
    DisableCOMInterrupt( PBX_SMDR_PORT );

    pp1.exit_ope_mode ();

    RTKTerminateTask( &thostess );

    halt_net ();

    uPrintf( "Done.\n" );
    exit( EXIT_SUCCESS );
    }

